# 文件读写

写入测试文件：

In [1]:

```py
%%writefile test.txt
this is a test file.
hello world!
python is good!
today is a good day.

```

```py
Writing test.txt

```

## 读文件

使用 `open` 函数或者 `file` 函数来读文件，使用文件名的字符串作为输入参数：

In [2]:

```py
f = open('test.txt')

```

In [3]:

```py
f = file('test.txt')

```

这两种方式没有太大区别。

默认以读的方式打开文件，如果文件不存在会报错。

可以使用 `read` 方法来读入文件中的所有内容：

In [4]:

```py
text = f.read()
print text

```

```py
this is a test file.
hello world!
python is good!
today is a good day.

```

也可以按照行读入内容，`readlines` 方法返回一个列表，每个元素代表文件中每一行的内容：

In [5]:

```py
f = open('test.txt')
lines = f.readlines()
print lines

```

```py
['this is a test file.\n', 'hello world!\n', 'python is good!\n', 'today is a good day.']

```

使用完文件之后，需要将文件关闭。

In [6]:

```py
f.close()

```

事实上，我们可以将 `f` 放在一个循环中，得到它每一行的内容：

In [7]:

```py
f = open('test.txt')
for line in f:
    print line
f.close()

```

```py
this is a test file.

hello world!

python is good!

today is a good day.

```

删除刚才创建的文件：

In [8]:

```py
import os
os.remove('test.txt')

```

## 写文件

我们使用 `open` 函数的写入模式来写文件：

In [9]:

```py
f = open('myfile.txt', 'w')
f.write('hello world!')
f.close()

```

使用 `w` 模式时，如果文件不存在会被创建，我们可以查看是否真的写入成功：

In [10]:

```py
print open('myfile.txt').read()

```

```py
hello world!

```

如果文件已经存在， `w` 模式会覆盖之前写的所有内容：

In [11]:

```py
f = open('myfile.txt', 'w')
f.write('another hello world!')
f.close()
print open('myfile.txt').read()

```

```py
another hello world!

```

除了写入模式，还有追加模式 `a` ，追加模式不会覆盖之前已经写入的内容，而是在之后继续写入：

In [12]:

```py
f = open('myfile.txt', 'a')
f.write('... and more')
f.close()
print open('myfile.txt').read()

```

```py
another hello world!... and more

```

写入结束之后一定要将文件关闭，否则可能出现内容没有完全写入文件中的情况。

还可以使用读写模式 `w+`：

In [13]:

```py
f = open('myfile.txt', 'w+')
f.write('hello world!')
f.seek(6)
print f.read()
f.close()

```

```py
world!

```

这里 `f.seek(6)` 移动到文件的第6个字符处，然后 `f.read()` 读出剩下的内容。

In [14]:

```py
import os
os.remove('myfile.txt')

```

## 二进制文件

二进制读写模式 b：

In [15]:

```py
import os
f = open('binary.bin', 'wb')
f.write(os.urandom(16))
f.close()

f = open('binary.bin', 'rb')
print repr(f.read())
f.close()

```

```py
'\x86H\x93\xe1\xd8\xef\xc0\xaa(\x17\xa9\xc9\xa51\xf1\x98'

```

In [16]:

```py
import os
os.remove('binary.bin')

```

## 换行符

不同操作系统的换行符可能不同：

*   `\r`
*   `\n`
*   `\r\n`

使用 `U` 选项，可以将这三个统一看成 `\n` 换行符。

## 关闭文件

在**Python**中，如果一个打开的文件不再被其他变量引用时，它会自动关闭这个文件。

所以正常情况下，如果一个文件正常被关闭了，忘记调用文件的 `close` 方法不会有什么问题。

关闭文件可以保证内容已经被写入文件，而不关闭可能会出现意想不到的结果：

In [17]:

```py
f = open('newfile.txt','w')
f.write('hello world')
g = open('newfile.txt', 'r')
print repr(g.read())

```

```py
''

```

虽然这里写了内容，但是在关闭之前，这个内容并没有被写入磁盘。

使用循环写入的内容也并不完整：

In [18]:

```py
f = open('newfile.txt','w')
for i in range(3000):
    f.write('hello world: ' + str(i) + '\n')

g = open('newfile.txt', 'r')
print g.read()
f.close()
g.close()

```

```py
hello world: 0
hello world: 1
hello world: 2
hello world: 3
hello world: 4
hello world: 5
hello world: 6
hello world: 7
hello world: 8
hello world: 9
hello world: 10
hello world: 11
hello world: 12
hello world: 13
hello world: 14
hello world: 15
hello world: 16
hello world: 17
hello world: 18
hello world: 19
hello world: 20
hello world: 21
hello world: 22
hello world: 23
hello world: 24
hello world: 25
hello world: 26
hello world: 27
hello world: 28
hello world: 29
hello world: 30
hello world: 31
hello world: 32
hello world: 33
hello world: 34
hello world: 35
hello world: 36
hello world: 37
hello world: 38
hello world: 39
hello world: 40
hello world: 41
hello world: 42
hello world: 43
hello world: 44
hello world: 45
hello world: 46
hello world: 47
hello world: 48
hello world: 49
hello world: 50
hello world: 51
hello world: 52
hello world: 53
hello world: 54
hello world: 55
hello world: 56
hello world: 57
hello world: 58
hello world: 59
hello world: 60
hello world: 61
hello world: 62
hello world: 63
hello world: 64
hello world: 65
hello world: 66
hello world: 67
hello world: 68
hello world: 69
hello world: 70
hello world: 71
hello world: 72
hello world: 73
hello world: 74
hello world: 75
hello world: 76
hello world: 77
hello world: 78
hello world: 79
hello world: 80
hello world: 81
hello world: 82
hello world: 83
hello world: 84
hello world: 85
hello world: 86
hello world: 87
hello world: 88
hello world: 89
hello world: 90
hello world: 91
hello world: 92
hello world: 93
hello world: 94
hello world: 95
hello world: 96
hello world: 97
hello world: 98
hello world: 99
hello world: 100
hello world: 101
hello world: 102
hello world: 103
hello world: 104
hello world: 105
hello world: 106
hello world: 107
hello world: 108
hello world: 109
hello world: 110
hello world: 111
hello world: 112
hello world: 113
hello world: 114
hello world: 115
hello world: 116
hello world: 117
hello world: 118
hello world: 119
hello world: 120
hello world: 121
hello world: 122
hello world: 123
hello world: 124
hello world: 125
hello world: 126
hello world: 127
hello world: 128
hello world: 129
hello world: 130
hello world: 131
hello world: 132
hello world: 133
hello world: 134
hello world: 135
hello world: 136
hello world: 137
hello world: 138
hello world: 139
hello world: 140
hello world: 141
hello world: 142
hello world: 143
hello world: 144
hello world: 145
hello world: 146
hello world: 147
hello world: 148
hello world: 149
hello world: 150
hello world: 151
hello world: 152
hello world: 153
hello world: 154
hello world: 155
hello world: 156
hello world: 157
hello world: 158
hello world: 159
hello world: 160
hello world: 161
hello world: 162
hello world: 163
hello world: 164
hello world: 165
hello world: 166
hello world: 167
hello world: 168
hello world: 169
hello world: 170
hello world: 171
hello world: 172
hello world: 173
hello world: 174
hello world: 175
hello world: 176
hello world: 177
hello world: 178
hello world: 179
hello world: 180
hello world: 181
hello world: 182
hello world: 183
hello world: 184
hello world: 185
hello world: 186
hello world: 187
hello world: 188
hello world: 189
hello world: 190
hello world: 191
hello world: 192
hello world: 193
hello world: 194
hello world: 195
hello world: 196
hello world: 197
hello world: 198
hello world: 199
hello world: 200
hello world: 201
hello world: 202
hello world: 203
hello world: 204
hello world: 205
hello world: 206
hello world: 207
hello world: 208
hello world: 209
hello world: 210
hello world: 211
hello world: 212
hello world: 213
hello world: 214
hello world: 215
hello world: 216
hello world: 217
hello world: 218
hello world: 219
hello world: 220
hello world: 221
hello world: 222
hello world: 223
hello world: 224
hello world: 225
hello world: 226
hello world: 227
hello world: 228
hello world: 229
hello world: 230
hello world: 231
hello world: 232
hello world: 233
hello world: 234
hello world: 235
hello world: 236
hello world: 237
hello world: 238
hello world: 239
hello world: 240
hello world: 241
hello world: 242
hello world: 243
hello world: 244
hello world: 245
hello world: 246
hello world: 247
hello world: 248
hello world: 249
hello world: 250
hello world: 251
hello world: 252
hello world: 253
hello world: 254
hello world: 255
hello world: 256
hello world: 257
hello world: 258
hello world: 259
hello world: 260
hello world: 261
hello world: 262
hello world: 263
hello world: 264
hello world: 265
hello world: 266
hello world: 267
hello world: 268
hello world: 269
hello world: 270
hello world: 271
hello world: 272
hello world: 273
hello world: 274
hello world: 275
hello world: 276
hello world: 277
hello world: 278
hello world: 279
hello world: 280
hello world: 281
hello world: 282
hello world: 283
hello world: 284
hello world: 285
hello world: 286
hello world: 287
hello world: 288
hello world: 289
hello world: 290
hello world: 291
hello world: 292
hello world: 293
hello world: 294
hello world: 295
hello world: 296
hello world: 297
hello world: 298
hello world: 299
hello world: 300
hello world: 301
hello world: 302
hello world: 303
hello world: 304
hello world: 305
hello world: 306
hello world: 307
hello world: 308
hello world: 309
hello world: 310
hello world: 311
hello world: 312
hello world: 313
hello world: 314
hello world: 315
hello world: 316
hello world: 317
hello world: 318
hello world: 319
hello world: 320
hello world: 321
hello world: 322
hello world: 323
hello world: 324
hello world: 325
hello world: 326
hello world: 327
hello world: 328
hello world: 329
hello world: 330
hello world: 331
hello world: 332
hello world: 333
hello world: 334
hello world: 335
hello world: 336
hello world: 337
hello world: 338
hello world: 339
hello world: 340
hello world: 341
hello world: 342
hello world: 343
hello world: 344
hello world: 345
hello world: 346
hello world: 347
hello world: 348
hello world: 349
hello world: 350
hello world: 351
hello world: 352
hello world: 353
hello world: 354
hello world: 355
hello world: 356
hello world: 357
hello world: 358
hello world: 359
hello world: 360
hello world: 361
hello world: 362
hello world: 363
hello world: 364
hello world: 365
hello world: 366
hello world: 367
hello world: 368
hello world: 369
hello world: 370
hello world: 371
hello world: 372
hello world: 373
hello world: 374
hello world: 375
hello world: 376
hello world: 377
hello world: 378
hello world: 379
hello world: 380
hello world: 381
hello world: 382
hello world: 383
hello world: 384
hello world: 385
hello world: 386
hello world: 387
hello world: 388
hello world: 389
hello world: 390
hello world: 391
hello world: 392
hello world: 393
hello world: 394
hello world: 395
hello world: 396
hello world: 397
hello world: 398
hello world: 399
hello world: 400
hello world: 401
hello world: 402
hello world: 403
hello world: 404
hello world: 405
hello world: 406
hello world: 407
hello world: 408
hello world: 409
hello world: 410
hello world: 411
hello world: 412
hello world: 413
hello world: 414
hello world: 415
hello world: 416
hello world: 417
hello world: 418
hello world: 419
hello world: 420
hello world: 421
hello world: 422
hello world: 423
hello world: 424
hello world: 425
hello world: 426
hello world: 427
hello world: 428
hello world: 429
hello world: 430
hello world: 431
hello world: 432
hello world: 433
hello world: 434
hello world: 435
hello world: 436
hello world: 437
hello world: 438
hello world: 439
hello world: 440
hello world: 441
hello world: 442
hello world: 443
hello world: 444
hello world: 445
hello world: 446
hello world: 447
hello world: 448
hello world: 449
hello world: 450
hello world: 451
hello world: 452
hello world: 453
hello world: 454
hello world: 455
hello world: 456
hello world: 457
hello world: 458
hello world: 459
hello world: 460
hello world: 461
hello world: 462
hello world: 463
hello world: 464
hello world: 465
hello world: 466
hello world: 467
hello world: 468
hello world: 469
hello world: 470
hello world: 471
hello world: 472
hello world: 473
hello world: 474
hello world: 475
hello world: 476
hello world: 477
hello world: 478
hello world: 479
hello world: 480
hello world: 481
hello world: 482
hello world: 483
hello world: 484
hello world: 485
hello world: 486
hello world: 487
hello world: 488
hello world: 489
hello world: 490
hello world: 491
hello world: 492
hello world: 493
hello world: 494
hello world: 495
hello world: 496
hello world: 497
hello world: 498
hello world: 499
hello world: 500
hello world: 501
hello world: 502
hello world: 503
hello world: 504
hello world: 505
hello world: 506
hello world: 507
hello world: 508
hello world: 509
hello world: 510
hello world: 511
hello world: 512
hello world: 513
hello world: 514
hello world: 515
hello world: 516
hello world: 517
hello world: 518
hello world: 519
hello world: 520
hello world: 521
hello world: 522
hello world: 523
hello world: 524
hello world: 525
hello world: 526
hello world: 527
hello world: 528
hello world: 529
hello world: 530
hello world: 531
hello world: 532
hello world: 533
hello world: 534
hello world: 535
hello world: 536
hello world: 537
hello world: 538
hello world: 539
hello world: 540
hello world: 541
hello world: 542
hello world: 543
hello world: 544
hello world: 545
hello world: 546
hello world: 547
hello world: 548
hello world: 549
hello world: 550
hello world: 551
hello world: 552
hello world: 553
hello world: 554
hello world: 555
hello world: 556
hello world: 557
hello world: 558
hello world: 559
hello world: 560
hello world: 561
hello world: 562
hello world: 563
hello world: 564
hello world: 565
hello world: 566
hello world: 567
hello world: 568
hello world: 569
hello world: 570
hello world: 571
hello world: 572
hello world: 573
hello world: 574
hello world: 575
hello world: 576
hello world: 577
hello world: 578
hello world: 579
hello world: 580
hello world: 581
hello world: 582
hello world: 583
hello world: 584
hello world: 585
hello world: 586
hello world: 587
hello world: 588
hello world: 589
hello world: 590
hello world: 591
hello world: 592
hello world: 593
hello world: 594
hello world: 595
hello world: 596
hello world: 597
hello world: 598
hello world: 599
hello world: 600
hello world: 601
hello world: 602
hello world: 603
hello world: 604
hello world: 605
hello world: 606
hello world: 607
hello world: 608
hello world: 609
hello world: 610
hello world: 611
hello world: 612
hello world: 613
hello world: 614
hello world: 615
hello world: 616
hello world: 617
hello world: 618
hello world: 619
hello world: 620
hello world: 621
hello world: 622
hello world: 623
hello world: 624
hello world: 625
hello world: 626
hello world: 627
hello world: 628
hello world: 629
hello world: 630
hello world: 631
hello world: 632
hello world: 633
hello world: 634
hello world: 635
hello world: 636
hello world: 637
hello world: 638
hello world: 639
hello world: 640
hello world: 641
hello world: 642
hello world: 643
hello world: 644
hello world: 645
hello world: 646
hello world: 647
hello world: 648
hello world: 649
hello world: 650
hello world: 651
hello world: 652
hello world: 653
hello world: 654
hello world: 655
hello world: 656
hello world: 657
hello world: 658
hello world: 659
hello world: 660
hello world: 661
hello world: 662
hello world: 663
hello world: 664
hello world: 665
hello world: 666
hello world: 667
hello world: 668
hello world: 669
hello world: 670
hello world: 671
hello world: 672
hello world: 673
hello world: 674
hello world: 675
hello world: 676
hello world: 677
hello world: 678
hello world: 679
hello world: 680
hello world: 681
hello world: 682
hello world: 683
hello world: 684
hello world: 685
hello world: 686
hello world: 687
hello world: 688
hello world: 689
hello world: 690
hello world: 691
hello world: 692
hello world: 693
hello world: 694
hello world: 695
hello world: 696
hello world: 697
hello world: 698
hello world: 699
hello world: 700
hello world: 701
hello world: 702
hello world: 703
hello world: 704
hello world: 705
hello world: 706
hello world: 707
hello world: 708
hello world: 709
hello world: 710
hello world: 711
hello world: 712
hello world: 713
hello world: 714
hello world: 715
hello world: 716
hello world: 717
hello world: 718
hello world: 719
hello world: 720
hello world: 721
hello world: 722
hello world: 723
hello world: 724
hello world: 725
hello world: 726
hello world: 727
hello world: 728
hello world: 729
hello world: 730
hello world: 731
hello world: 732
hello world: 733
hello world: 734
hello world: 735
hello world: 736
hello world: 737
hello world: 738
hello world: 739
hello world: 740
hello world: 741
hello world: 742
hello world: 743
hello world: 744
hello world: 745
hello world: 746
hello world: 747
hello world: 748
hello world: 749
hello world: 750
hello world: 751
hello world: 752
hello world: 753
hello world: 754
hello world: 755
hello world: 756
hello world: 757
hello world: 758
hello world: 759
hello world: 760
hello world: 761
hello world: 762
hello world: 763
hello world: 764
hello world: 765
hello world: 766
hello world: 767
hello world: 768
hello world: 769
hello world: 770
hello world: 771
hello world: 772
hello world: 773
hello world: 774
hello world: 775
hello world: 776
hello world: 777
hello world: 778
hello world: 779
hello world: 780
hello world: 781
hello world: 782
hello world: 783
hello world: 784
hello world: 785
hello world: 786
hello world: 787
hello world: 788
hello world: 789
hello world: 790
hello world: 791
hello world: 792
hello world: 793
hello world: 794
hello world: 795
hello world: 796
hello world: 797
hello world: 798
hello world: 799
hello world: 800
hello world: 801
hello world: 802
hello world: 803
hello world: 804
hello world: 805
hello world: 806
hello world: 807
hello world: 808
hello world: 809
hello world: 810
hello world: 811
hello world: 812
hello world: 813
hello world: 814
hello world: 815
hello world: 816
hello world: 817
hello world: 818
hello world: 819
hello world: 820
hello world: 821
hello world: 822
hello world: 823
hello world: 824
hello world: 825
hello world: 826
hello world: 827
hello world: 828
hello world: 829
hello world: 830
hello world: 831
hello world: 832
hello world: 833
hello world: 834
hello world: 835
hello world: 836
hello world: 837
hello world: 838
hello world: 839
hello world: 840
hello world: 841
hello world: 842
hello world: 843
hello world: 844
hello world: 845
hello world: 846
hello world: 847
hello world: 848
hello world: 849
hello world: 850
hello world: 851
hello world: 852
hello world: 853
hello world: 854
hello world: 855
hello world: 856
hello world: 857
hello world: 858
hello world: 859
hello world: 860
hello world: 861
hello world: 862
hello world: 863
hello world: 864
hello world: 865
hello world: 866
hello world: 867
hello world: 868
hello world: 869
hello world: 870
hello world: 871
hello world: 872
hello world: 873
hello world: 874
hello world: 875
hello world: 876
hello world: 877
hello world: 878
hello world: 879
hello world: 880
hello world: 881
hello world: 882
hello world: 883
hello world: 884
hello world: 885
hello world: 886
hello world: 887
hello world: 888
hello world: 889
hello world: 890
hello world: 891
hello world: 892
hello world: 893
hello world: 894
hello world: 895
hello world: 896
hello world: 897
hello world: 898
hello world: 899
hello world: 900
hello world: 901
hello world: 902
hello world: 903
hello world: 904
hello world: 905
hello world: 906
hello world: 907
hello world: 908
hello world: 909
hello world: 910
hello world: 911
hello world: 912
hello world: 913
hello world: 914
hello world: 915
hello world: 916
hello world: 917
hello world: 918
hello world: 919
hello world: 920
hello world: 921
hello world: 922
hello world: 923
hello world: 924
hello world: 925
hello world: 926
hello world: 927
hello world: 928
hello world: 929
hello world: 930
hello world: 931
hello world: 932
hello world: 933
hello world: 934
hello world: 935
hello world: 936
hello world: 937
hello world: 938
hello world: 939
hello world: 940
hello world: 941
hello world: 942
hello world: 943
hello world: 944
hello world: 945
hello world: 946
hello world: 947
hello world: 948
hello world: 949
hello world: 950
hello world: 951
hello world: 952
hello world: 953
hello world: 954
hello world: 955
hello world: 956
hello world: 957
hello world: 958
hello world: 959
hello world: 960
hello world: 961
hello world: 962
hello world: 963
hello world: 964
hello world: 965
hello world: 966
hello world: 967
hello world: 968
hello world: 969
hello world: 970
hello world: 971
hello world: 972
hello world: 973
hello world: 974
hello world: 975
hello world: 976
hello world: 977
hello world: 978
hello world: 979
hello world: 980
hello world: 981
hello world: 982
hello world: 983
hello world: 984
hello world: 985
hello world: 986
hello world: 987
hello world: 988
hello world: 989
hello world: 990
hello world: 991
hello world: 992
hello world: 993
hello world: 994
hello world: 995
hello world: 996
hello world: 997
hello world: 998
hello world: 999
hello world: 1000
hello world: 1001
hello world: 1002
hello world: 1003
hello world: 1004
hello world: 1005
hello world: 1006
hello world: 1007
hello world: 1008
hello world: 1009
hello world: 1010
hello world: 1011
hello world: 1012
hello world: 1013
hello world: 1014
hello world: 1015
hello world: 1016
hello world: 1017
hello world: 1018
hello world: 1019
hello world: 1020
hello world: 1021
hello world: 1022
hello world: 1023
hello world: 1024
hello world: 1025
hello world: 1026
hello world: 1027
hello world: 1028
hello world: 1029
hello world: 1030
hello world: 1031
hello world: 1032
hello world: 1033
hello world: 1034
hello world: 1035
hello world: 1036
hello world: 1037
hello world: 1038
hello world: 1039
hello world: 1040
hello world: 1041
hello world: 1042
hello world: 1043
hello world: 1044
hello world: 1045
hello world: 1046
hello world: 1047
hello world: 1048
hello world: 1049
hello world: 1050
hello world: 1051
hello world: 1052
hello world: 1053
hello world: 1054
hello world: 1055
hello world: 1056
hello world: 1057
hello world: 1058
hello world: 1059
hello world: 1060
hello world: 1061
hello world: 1062
hello world: 1063
hello world: 1064
hello world: 1065
hello world: 1066
hello world: 1067
hello world: 1068
hello world: 1069
hello world: 1070
hello world: 1071
hello world: 1072
hello world: 1073
hello world: 1074
hello world: 1075
hello world: 1076
hello world: 1077
hello world: 1078
hello world: 1079
hello world: 1080
hello world: 1081
hello world: 1082
hello world: 1083
hello world: 1084
hello world: 1085
hello world: 1086
hello world: 1087
hello world: 1088
hello world: 1089
hello world: 1090
hello world: 1091
hello world: 1092
hello world: 1093
hello world: 1094
hello world: 1095
hello world: 1096
hello world: 1097
hello world: 1098
hello world: 1099
hello world: 1100
hello world: 1101
hello world: 1102
hello world: 1103
hello world: 1104
hello world: 1105
hello world: 1106
hello world: 1107
hello world: 1108
hello world: 1109
hello world: 1110
hello world: 1111
hello world: 1112
hello world: 1113
hello world: 1114
hello world: 1115
hello world: 1116
hello world: 1117
hello world: 1118
hello world: 1119
hello world: 1120
hello world: 1121
hello world: 1122
hello world: 1123
hello world: 1124
hello world: 1125
hello world: 1126
hello world: 1127
hello world: 1128
hello world: 1129
hello world: 1130
hello world: 1131
hello world: 1132
hello world: 1133
hello world: 1134
hello world: 1135
hello world: 1136
hello world: 1137
hello world: 1138
hello world: 1139
hello world: 1140
hello world: 1141
hello world: 1142
hello world: 1143
hello world: 1144
hello world: 1145
hello world: 1146
hello world: 1147
hello world: 1148
hello world: 1149
hello world: 1150
hello world: 1151
hello world: 1152
hello world: 1153
hello world: 1154
hello world: 1155
hello world: 1156
hello world: 1157
hello world: 1158
hello world: 1159
hello world: 1160
hello world: 1161
hello world: 1162
hello world: 1163
hello world: 1164
hello world: 1165
hello world: 1166
hello world: 1167
hello world: 1168
hello world: 1169
hello world: 1170
hello world: 1171
hello world: 1172
hello world: 1173
hello world: 1174
hello world: 1175
hello world: 1176
hello world: 1177
hello world: 1178
hello world: 1179
hello world: 1180
hello world: 1181
hello world: 1182
hello world: 1183
hello world: 1184
hello world: 1185
hello world: 1186
hello world: 1187
hello world: 1188
hello world: 1189
hello world: 1190
hello world: 1191
hello world: 1192
hello world: 1193
hello world: 1194
hello world: 1195
hello world: 1196
hello world: 1197
hello world: 1198
hello world: 1199
hello world: 1200
hello world: 1201
hello world: 1202
hello world: 1203
hello world: 1204
hello world: 1205
hello world: 1206
hello world: 1207
hello world: 1208
hello world: 1209
hello world: 1210
hello world: 1211
hello world: 1212
hello world: 1213
hello world: 1214
hello world: 1215
hello world: 1216
hello world: 1217
hello world: 1218
hello world: 1219
hello world: 1220
hello world: 1221
hello world: 1222
hello world: 1223
hello world: 1224
hello world: 1225
hello world: 1226
hello world: 1227
hello world: 1228
hello world: 1229
hello world: 1230
hello world: 1231
hello world: 1232
hello world: 1233
hello world: 1234
hello world: 1235
hello world: 1236
hello world: 1237
hello world: 1238
hello world: 1239
hello world: 1240
hello world: 1241
hello world: 1242
hello world: 1243
hello world: 1244
hello world: 1245
hello world: 1246
hello world: 1247
hello world: 1248
hello world: 1249
hello world: 1250
hello world: 1251
hello world: 1252
hello world: 1253
hello world: 1254
hello world: 1255
hello world: 1256
hello world: 1257
hello world: 1258
hello world: 1259
hello world: 1260
hello world: 1261
hello world: 1262
hello world: 1263
hello world: 1264
hello world: 1265
hello world: 1266
hello world: 1267
hello world: 1268
hello world: 1269
hello world: 1270
hello world: 1271
hello world: 1272
hello world: 1273
hello world: 1274
hello world: 1275
hello world: 1276
hello world: 1277
hello world: 1278
hello world: 1279
hello world: 1280
hello world: 1281
hello world: 1282
hello world: 1283
hello world: 1284
hello world: 1285
hello world: 1286
hello world: 1287
hello world: 1288
hello world: 1289
hello world: 1290
hello world: 1291
hello world: 1292
hello world: 1293
hello world: 1294
hello world: 1295
hello world: 1296
hello world: 1297
hello world: 1298
hello world: 1299
hello world: 1300
hello world: 1301
hello world: 1302
hello world: 1303
hello world: 1304
hello world: 1305
hello world: 1306
hello world: 1307
hello world: 1308
hello world: 1309
hello world: 1310
hello world: 1311
hello world: 1312
hello world: 1313
hello world: 1314
hello world: 1315
hello world: 1316
hello world: 1317
hello world: 1318
hello world: 1319
hello world: 1320
hello world: 1321
hello world: 1322
hello world: 1323
hello world: 1324
hello world: 1325
hello world: 1326
hello world: 1327
hello world: 1328
hello world: 1329
hello world: 1330
hello world: 1331
hello world: 1332
hello world: 1333
hello world: 1334
hello world: 1335
hello world: 1336
hello world: 1337
hello world: 1338
hello world: 1339
hello world: 1340
hello world: 1341
hello world: 1342
hello world: 1343
hello world: 1344
hello world: 1345
hello world: 1346
hello world: 1347
hello world: 1348
hello world: 1349
hello world: 1350
hello world: 1351
hello world: 1352
hello world: 1353
hello world: 1354
hello world: 1355
hello world: 1356
hello world: 1357
hello world: 1358
hello world: 1359
hello world: 1360
hello world: 1361
hello world: 1362
hello world: 1363
hello world: 1364
hello world: 1365
hello world: 1366
hello world: 1367
hello world: 1368
hello world: 1369
hello world: 1370
hello world: 1371
hello world: 1372
hello world: 1373
hello world: 1374
hello world: 1375
hello world: 1376
hello world: 1377
hello world: 1378
hello world: 1379
hello world: 1380
hello world: 1381
hello world: 1382
hello world: 1383
hello world: 1384
hello world: 1385
hello world: 1386
hello world: 1387
hello world: 1388
hello world: 1389
hello world: 1390
hello world: 1391
hello world: 1392
hello world: 1393
hello world: 1394
hello world: 1395
hello world: 1396
hello world: 1397
hello world: 1398
hello world: 1399
hello world: 1400
hello world: 1401
hello world: 1402
hello world: 1403
hello world: 1404
hello world: 1405
hello world: 1406
hello world: 1407
hello world: 1408
hello world: 1409
hello world: 1410
hello world: 1411
hello world: 1412
hello world: 1413
hello world: 1414
hello world: 1415
hello world: 1416
hello world: 1417
hello world: 1418
hello world: 1419
hello world: 1420
hello world: 1421
hello world: 1422
hello world: 1423
hello world: 1424
hello world: 1425
hello world: 1426
hello world: 1427
hello world: 1428
hello world: 1429
hello world: 1430
hello world: 1431
hello world: 1432
hello world: 1433
hello world: 1434
hello world: 1435
hello world: 1436
hello world: 1437
hello world: 1438
hello world: 1439
hello world: 1440
hello world: 1441
hello world: 1442
hello world: 1443
hello world: 1444
hello world: 1445
hello world: 1446
hello world: 1447
hello world: 1448
hello world: 1449
hello world: 1450
hello world: 1451
hello world: 1452
hello world: 1453
hello world: 1454
hello world: 1455
hello world: 1456
hello world: 1457
hello world: 1458
hello world: 1459
hello world: 1460
hello world: 1461
hello world: 1462
hello world: 1463
hello world: 1464
hello world: 1465
hello world: 1466
hello world: 1467
hello world: 1468
hello world: 1469
hello world: 1470
hello world: 1471
hello world: 1472
hello world: 1473
hello world: 1474
hello world: 1475
hello world: 1476
hello world: 1477
hello world: 1478
hello world: 1479
hello world: 1480
hello world: 1481
hello world: 1482
hello world: 1483
hello world: 1484
hello world: 1485
hello world: 1486
hello world: 1487
hello world: 1488
hello world: 1489
hello world: 1490
hello world: 1491
hello world: 1492
hello world: 1493
hello world: 1494
hello world: 1495
hello world: 1496
hello world: 1497
hello world: 1498
hello world: 1499
hello world: 1500
hello world: 1501
hello world: 1502
hello world: 1503
hello world: 1504
hello world: 1505
hello world: 1506
hello world: 1507
hello world: 1508
hello world: 1509
hello world: 1510
hello world: 1511
hello world: 1512
hello world: 1513
hello world: 1514
hello world: 1515
hello world: 1516
hello world: 1517
hello world: 1518
hello world: 1519
hello world: 1520
hello world: 1521
hello world: 1522
hello world: 1523
hello world: 1524
hello world: 1525
hello world: 1526
hello world: 1527
hello world: 1528
hello world: 1529
hello world: 1530
hello world: 1531
hello world: 1532
hello world: 1533
hello world: 1534
hello world: 1535
hello world: 1536
hello world: 1537
hello world: 1538
hello world: 1539
hello world: 1540
hello world: 1541
hello world: 1542
hello world: 1543
hello world: 1544
hello world: 1545
hello world: 1546
hello world: 1547
hello world: 1548
hello world: 1549
hello world: 1550
hello world: 1551
hello world: 1552
hello world: 1553
hello world: 1554
hello world: 1555
hello world: 1556
hello world: 1557
hello world: 1558
hello world: 1559
hello world: 1560
hello world: 1561
hello world: 1562
hello world: 1563
hello world: 1564
hello world: 1565
hello world: 1566
hello world: 1567
hello world: 1568
hello world: 1569
hello world: 1570
hello world: 1571
hello world: 1572
hello world: 1573
hello world: 1574
hello world: 1575
hello world: 1576
hello world: 1577
hello world: 1578
hello world: 1579
hello world: 1580
hello world: 1581
hello world: 1582
hello world: 1583
hello world: 1584
hello world: 1585
hello world: 1586
hello world: 1587
hello world: 1588
hello world: 1589
hello world: 1590
hello world: 1591
hello world: 1592
hello world: 1593
hello world: 1594
hello world: 1595
hello world: 1596
hello world: 1597
hello world: 1598
hello world: 1599
hello world: 1600
hello world: 1601
hello world: 1602
hello world: 1603
hello world: 1604
hello world: 1605
hello world: 1606
hello world: 1607
hello world: 1608
hello world: 1609
hello world: 1610
hello world: 1611
hello world: 1612
hello world: 1613
hello world: 1614
hello world: 1615
hello world: 1616
hello world: 1617
hello world: 1618
hello world: 1619
hello world: 1620
hello world: 1621
hello world: 1622
hello world: 1623
hello world: 1624
hello world: 1625
hello world: 1626
hello world: 1627
hello world: 1628
hello world: 1629
hello world: 1630
hello world: 1631
hello world: 1632
hello world: 1633
hello world: 1634
hello world: 1635
hello world: 1636
hello world: 1637
hello world: 1638
hello world: 1639
hello world: 1640
hello world: 1641
hello world: 1642
hello world: 1643
hello world: 1644
hello world: 1645
hello world: 1646
hello world: 1647
hello world: 1648
hello world: 1649
hello world: 1650
hello world: 1651
hello world: 1652
hello world: 1653
hello world: 1654
hello world: 1655
hello world: 1656
hello world: 1657
hello world: 1658
hello world: 1659
hello world: 1660
hello world: 1661
hello world: 1662
hello world: 1663
hello world: 1664
hello world: 1665
hello world: 1666
hello world: 1667
hello world: 1668
hello world: 1669
hello world: 1670
hello world: 1671
hello world: 1672
hello world: 1673
hello world: 1674
hello world: 1675
hello world: 1676
hello world: 1677
hello world: 1678
hello world: 1679
hello world: 1680
hello world: 1681
hello world: 1682
hello world: 1683
hello world: 1684
hello world: 1685
hello world: 1686
hello world: 1687
hello world: 1688
hello world: 1689
hello world: 1690
hello world: 1691
hello world: 1692
hello world: 1693
hello world: 1694
hello world: 1695
hello world: 1696
hello world: 1697
hello world: 1698
hello world: 1699
hello world: 1700
hello world: 1701
hello world: 1702
hello world: 1703
hello world: 1704
hello world: 1705
hello world: 1706
hello world: 1707
hello world: 1708
hello world: 1709
hello world: 1710
hello world: 1711
hello world: 1712
hello world: 1713
hello world: 1714
hello world: 1715
hello world: 1716
hello world: 1717
hello world: 1718
hello world: 1719
hello world: 1720
hello world: 1721
hello world: 1722
hello world: 1723
hello world: 1724
hello world: 1725
hello world: 1726
hello world: 1727
hello world: 1728
hello world: 1729
hello world: 1730
hello world: 1731
hello world: 1732
hello world: 1733
hello world: 1734
hello world: 1735
hello world: 1736
hello world: 1737
hello world: 1738
hello world: 1739
hello world: 1740
hello world: 1741
hello world: 1742
hello world: 1743
hello world: 1744
hello world: 1745
hello world: 1746
hello world: 1747
hello world: 1748
hello world: 1749
hello world: 1750
hello world: 1751
hello world: 1752
hello world: 1753
hello world: 1754
hello world: 1755
hello world: 1756
hello world: 1757
hello world: 1758
hello world: 1759
hello world: 1760
hello world: 1761
hello world: 1762
hello world: 1763
hello world: 1764
hello world: 1765
hello world: 1766
hello world: 1767
hello world: 1768
hello world: 1769
hello world: 1770
hello world: 1771
hello world: 1772
hello world: 1773
hello world: 1774
hello world: 1775
hello world: 1776
hello world: 1777
hello world: 1778
hello world: 1779
hello world: 1780
hello world: 1781
hello world: 1782
hello world: 1783
hello world: 1784
hello world: 1785
hello world: 1786
hello world: 1787
hello world: 1788
hello world: 1789
hello world: 1790
hello world: 1791
hello world: 1792
hello world: 1793
hello world: 1794
hello world: 1795
hello world: 1796
hello world: 1797
hello world: 1798
hello world: 1799
hello world: 1800
hello world: 1801
hello world: 1802
hello world: 1803
hello world: 1804
hello world: 1805
hello world: 1806
hello world: 1807
hello world: 1808
hello world: 1809
hello world: 1810
hello world: 1811
hello world: 1812
hello world: 1813
hello world: 1814
hello world: 1815
hello world: 1816
hello world: 1817
hello world: 1818
hello world: 1819
hello world: 1820
hello world: 1821
hello world: 1822
hello world: 1823
hello world: 1824
hello world: 1825
hello world: 1826
hello world: 1827
hello world: 1828
hello world: 1829
hello world: 1830
hello world: 1831
hello world: 1832
hello world: 1833
hello world: 1834
hello world: 1835
hello world: 1836
hello world: 1837
hello world: 1838
hello world: 1839
hello world: 1840
hello world: 1841
hello world: 1842
hello world: 1843
hello world: 1844
hello world: 1845
hello world: 1846
hello world: 1847
hello world: 1848
hello world: 1849
hello world: 1850
hello world: 1851
hello world: 1852
hello world: 1853
hello world: 1854
hello world: 1855
hello world: 1856
hello world: 1857
hello world: 1858
hello world: 1859
hello world: 1860
hello world: 1861
hello world: 1862
hello world: 1863
hello world: 1864
hello world: 1865
hello world: 1866
hello world: 1867
hello world: 1868
hello world: 1869
hello world: 1870
hello world: 1871
hello world: 1872
hello world: 1873
hello world: 1874
hello world: 1875
hello world: 1876
hello world: 1877
hello world: 1878
hello world: 1879
hello world: 1880
hello world: 1881
hello world: 1882
hello world: 1883
hello world: 1884
hello world: 1885
hello world: 1886
hello world: 1887
hello world: 1888
hello world: 1889
hello world: 1890
hello world: 1891
hello world: 1892
hello world: 1893
hello world: 1894
hello world: 1895
hello world: 1896
hello world: 1897
hello world: 1898
hello world: 1899
hello world: 1900
hello world: 1901
hello world: 1902
hello world: 1903
hello world: 1904
hello world: 1905
hello world: 1906
hello world: 1907
hello world: 1908
hello world: 1909
hello world: 1910
hello world: 1911
hello world: 1912
hello world: 1913
hello world: 1914
hello world: 1915
hello world: 1916
hello world: 1917
hello world: 1918
hello world: 1919
hello world: 1920
hello world: 1921
hello world: 1922
hello world: 1923
hello world: 1924
hello world: 1925
hello world: 1926
hello world: 1927
hello world: 1928
hello world: 1929
hello world: 1930
hello world: 1931
hello world: 1932
hello world: 1933
hello world: 1934
hello world: 1935
hello world: 1936
hello world: 1937
hello world: 1938
hello world: 1939
hello world: 1940
hello world: 1941
hello world: 1942
hello world: 1943
hello world: 1944
hello world: 1945
hello world: 1946
hello world: 1947
hello world: 1948
hello world: 1949
hello world: 1950
hello world: 1951
hello world: 1952
hello world: 1953
hello world: 1954
hello world: 1955
hello world: 1956
hello world: 1957
hello world: 1958
hello world: 1959
hello world: 1960
hello world: 1961
hello world: 1962
hello world: 1963
hello world: 1964
hello world: 1965
hello world: 1966
hello world: 1967
hello world: 1968
hello world: 1969
hello world: 1970
hello world: 1971
hello world: 1972
hello world: 1973
hello world: 1974
hello world: 1975
hello world: 1976
hello world: 1977
hello world: 1978
hello world: 1979
hello world: 1980
hello world: 1981
hello world: 1982
hello world: 1983
hello world: 1984
hello world: 1985
hello world: 1986
hello world: 1987
hello world: 1988
hello world: 1989
hello world: 1990
hello world: 1991
hello world: 1992
hello world: 1993
hello world: 1994
hello world: 1995
hello world: 1996
hello world: 1997
hello world: 1998
hello world: 1999
hello world: 2000
hello world: 2001
hello world: 2002
hello world: 2003
hello world: 2004
hello world: 2005
hello world: 2006
hello world: 2007
hello world: 2008
hello world: 2009
hello world: 2010
hello world: 2011
hello world: 2012
hello world: 2013
hello world: 2014
hello world: 2015
hello world: 2016
hello world: 2017
hello world: 2018
hello world: 2019
hello world: 2020
hello world: 2021
hello world: 2022
hello world: 2023
hello world: 2024
hello world: 2025
hello world: 2026
hello world: 2027
hello world: 2028
hello world: 2029
hello world: 2030
hello world: 2031
hello world: 2032
hello world: 2033
hello world: 2034
hello world: 2035
hello world: 2036
hello world: 2037
hello world: 2038
hello world: 2039
hello world: 2040
hello world: 2041
hello world: 2042
hello world: 2043
hello world: 2044
hello world: 2045
hello world: 2046
hello world: 2047
hello world: 2048
hello world: 2049
hello world: 2050
hello world: 2051
hello world: 2052
hello world: 2053
hello world: 2054
hello world: 2055
hello world: 2056
hello world: 2057
hello world: 2058
hello world: 2059
hello world: 2060
hello world: 2061
hello world: 2062
hello world: 2063
hello world: 2064
hello world: 2065
hello world: 2066
hello world: 2067
hello world: 2068
hello world: 2069
hello world: 2070
hello world: 2071
hello world: 2072
hello world: 2073
hello world: 2074
hello world: 2075
hello world: 2076
hello world: 2077
hello world: 2078
hello world: 2079
hello world: 2080
hello world: 2081
hello world: 2082
hello world: 2083
hello world: 2084
hello world: 2085
hello world: 2086
hello world: 2087
hello world: 2088
hello world: 2089
hello world: 2090
hello world: 2091
hello world: 2092
hello world: 2093
hello world: 2094
hello world: 2095
hello world: 2096
hello world: 2097
hello world: 2098
hello world: 2099
hello world: 2100
hello world: 2101
hello world: 2102
hello world: 2103
hello world: 2104
hello world: 2105
hello world: 2106
hello world: 2107
hello world: 2108
hello world: 2109
hello world: 2110
hello world: 2111
hello world: 2112
hello world: 2113
hello world: 2114
hello world: 2115
hello world: 2116
hello world: 2117
hello world: 2118
hello world: 2119
hello world: 2120
hello world: 2121
hello world: 2122
hello world: 2123
hello world: 2124
hello world: 2125
hello world: 2126
hello world: 2127
hello world: 2128
hello world: 2129
hello world: 2130
hello world: 2131
hello world: 2132
hello world: 2133
hello world: 2134
hello world: 2135
hello world: 2136
hello world: 2137
hello world: 2138
hello world: 2139
hello world: 2140
hello world: 2141
hello world: 2142
hello world: 2143
hello world: 2144
hello world: 2145
hello world: 2146
hello world: 2147
hello world: 2148
hello world: 2149
hello world: 2150
hello world: 2151
hello world: 2152
hello world: 2153
hello world: 2154
hello world: 2155
hello world: 2156
hello world: 2157
hello world: 2158
hello world: 2159
hello world: 2160
hello world: 2161
hello world: 2162
hello world: 2163
hello world: 2164
hello world: 2165
hello world: 2166
hello world: 2167
hello world: 2168
hello world: 2169
hello world: 2170
hello world: 2171
hello world: 2172
hello world: 2173
hello world: 2174
hello world: 2175
hello world: 2176
hello world: 2177
hello world: 2178
hello world: 2179
hello world: 2180
hello world: 2181
hello world: 2182
hello world: 2183
hello world: 2184
hello world: 2185
hello world: 2186
hello world: 2187
hello world: 2188
hello world: 2189
hello world: 2190
hello world: 2191
hello world: 2192
hello world: 2193
hello world: 2194
hello world: 2195
hello world: 2196
hello world: 2197
hello world: 2198
hello world: 2199
hello world: 2200
hello world: 2201
hello world: 2202
hello world: 2203
hello world: 2204
hello world: 2205
hello world: 2206
hello world: 2207
hello world: 2208
hello world: 2209
hello world: 2210
hello world: 2211
hello world: 2212
hello world: 2213
hello world: 2214
hello world: 2215
hello world: 2216
hello world: 2217
hello world: 2218
hello world: 2219
hello world: 2220
hello world: 2221
hello world: 2222
hello world: 2223
hello world: 2224
hello world: 2225
hello world: 2226
hello world: 2227
hello world: 2228
hello world: 2229
hello world: 2230
hello world: 2231
hello world: 2232
hello world: 2233
hello world: 2234
hello world: 2235
hello world: 2236
hello world: 2237
hello world: 2238
hello world: 2239
hello world: 2240
hello world: 2241
hello world: 2242
hello world: 2243
hello world: 2244
hello world: 2245
hello world: 2246
hello world: 2247
hello world: 2248
hello world: 2249
hello world: 2250
hello world: 2251
hello world: 2252
hello world: 2253
hello world: 2254
hello world: 2255
hello world: 2256
hello world: 2257
hello world: 2258
hello world: 2259
hello world: 2260
hello world: 2261
hello world: 2262
hello world: 2263
hello world: 2264
hello world: 2265
hello world: 2266
hello world: 2267
hello world: 2268
hello world: 2269
hello world: 2270
hello world: 2271
hello world: 2272
hello world: 2273
hello world: 2274
hello world: 2275
hello world: 2276
hello world: 2277
hello world: 2278
hello world: 2279
hello world: 2280
hello world: 2281
hello world: 2282
hello world: 2283
hello world: 2284
hello world: 2285
hello world: 2286
hello world: 2287
hello world: 2288
hello world: 2289
hello world: 2290
hello world: 2291
hello world: 2292
hello world: 2293
hello world: 2294
hello world: 2295
hello world: 2296
hello world: 2297
hello world: 2298
hello world: 2299
hello world: 2300
hello world: 2301
hello world: 2302
hello world: 2303
hello world: 2304
hello world: 2305
hello world: 2306
hello world: 2307
hello world: 2308
hello world: 2309
hello world: 2310
hello world: 2311
hello world: 2312
hello world: 2313
hello world: 2314
hello world: 2315
hello world: 2316
hello world: 2317
hello world: 2318
hello world: 2319
hello world: 2320
hello world: 2321
hello world: 2322
hello world: 2323
hello world: 2324
hello world: 2325
hello world: 2326
hello world: 2327
hello world: 2328
hello world: 2329
hello world: 2330
hello world: 2331
hello world: 2332
hello world: 2333
hello world: 2334
hello world: 2335
hello world: 2336
hello world: 2337
hello world: 2338
hello world: 2339
hello world: 2340
hello world: 2341
hello world: 2342
hello world: 2343
hello world: 2344
hello world: 2345
hello world: 2346
hello world: 2347
hello world: 2348
hello world: 2349
hello world: 2350
hello world: 2351
hello world: 2352
hello world: 2353
hello world: 2354
hello world: 2355
hello world: 2356
hello world: 2357
hello world: 2358
hello world: 2359
hello world: 2360
hello world: 2361
hello world: 2362
hello world: 2363
hello world: 2364
hello world: 2365
hello world: 2366
hello world: 2367
hello world: 2368
hello world: 2369
hello world: 2370
hello world: 2371
hello world: 2372
hello world: 2373
hello world: 2374
hello world: 2375
hello world: 2376
hello world: 2377
hello world: 2378
hello world: 2379
hello world: 2380
hello world: 2381
hello world: 2382
hello world: 2383
hello world: 2384
hello world: 2385
hello world: 2386
hello world: 2387
hello world: 2388
hello world: 2389
hello world: 2390
hello world: 2391
hello world: 2392
hello world: 2393
hello world: 2394
hello world: 2395
hello world: 2396
hello world: 2397
hello world: 2398
hello world: 2399
hello world: 2400
hello world: 2401
hello world: 2402
hello world: 2403
hello world: 2404
hello world: 2405
hello world: 2406
hello world: 2407
hello world: 2408
hello world: 2409
hello world: 2410
hello world: 2411
hello world: 2412
hello world: 2413
hello world: 2414
hello world: 2415
hello world: 2416
hello world: 2417
hello world: 2418
hello world: 2419
hello world: 2420
hello world: 2421
hello world: 2422
hello world: 2423
hello world: 2424
hello world: 2425
hello world: 2426
hello world: 2427
hello world: 2428
hello world: 2429
hello world: 2430
hello world: 2431
hello world: 2432
hello world: 2433
hello world: 2434
hello world: 2435
hello world: 2436
hello world: 2437
hello world: 2438
hello world: 2439
hello world: 2440
hello world: 2441
hello world: 2442
hello world: 2443
hello world: 2444
hello world: 2445
hello world: 2446
hello world: 2447
hello world: 2448
hello world: 2449
hello world: 2450
hello world: 2451
hello world: 2452
hello world: 2453
hello world: 2454
hello world: 2455
hello world: 2456
hello world: 2457
hello world: 2458
hello world: 2459
hello world: 2460
hello world: 2461
hello world: 2462
hello world: 2463
hello world: 2464
hello world: 2465
hello world: 2466
hello world: 2467
hello world: 2468
hello world: 2469
hello world: 2470
hello world: 2471
hello world: 2472
hello world: 2473
hello world: 2474
hello world: 2475
hello world: 2476
hello world: 2477
hello world: 2478
hello world: 2479
hello world: 2480
hello world: 2481
hello world: 2482
hello world: 2483
hello world: 2484
hello world: 2485
hello world: 2486
hello world: 2487
hello world: 2488
hello world: 2489
hello world: 2490
hello world: 2491
hello world: 2492
hello world: 2493
hello world: 2494
hello world: 2495
hello world: 2496
hello world: 2497
hello world: 2498
hello world: 2499
hello world: 2500
hello world: 2501
hello world: 2502
hello world: 2503
hello world: 2504
hello world: 2505
hello world: 2506
hello world: 2507
hello world: 2508
hello world: 2509
hello world: 2510
hello world: 2511
hello world: 2512
hello world: 2513
hello world: 2514
hello world: 2515
hello world: 2516
hello world: 2517
hello world: 2518
hello world: 2519
hello world: 2520
hello world: 2521
hello world: 2522
hello world: 2523
hello world: 2524
hello world: 2525
hello world: 2526
hello world: 2527
hello world: 2528
hello world: 2529
hello world: 2530
hello world: 2531
hello world: 2532
hello world: 2533
hello world: 2534
hello world: 2535
hello world: 2536
hello world: 2537
hello world: 2538
hello world: 2539
hello world: 2540
hello world: 2541
hello world: 2542
hello world: 2543
hello world: 2544
hello world: 2545
hello world: 2546
hello world: 2547
hello world: 2548
hello world: 2549
hello world: 2550
hello world: 2551
hello world: 2552
hello world: 2553
hello world: 2554
hello world: 2555
hello world: 2556
hello world: 2557
hello world: 2558
hello world: 2559
hello world: 2560
hello world: 2561
hello world: 2562
hello world: 2563
hello world: 2564
hello world: 2565
hello world: 2566
hello world: 2567
hello world: 2568
hello world: 2569
hello world: 2570
hello world: 2571
hello world: 2572
hello world: 2573
hello world: 2574
hello world: 2575
hello world: 2576
hello world: 2577
hello world: 2578
hello world: 2579
hello world: 2580
hello world: 2581
hello world: 2582
hello world: 2583
hello world: 2584
hello world: 2585
hello world: 2586
hello world: 2587
hello world: 2588
hello world: 2589
hello world: 2590
hello world: 2591
hello world: 2592
hello world: 2593
hello world: 2594
hello world: 2595
hello world: 2596
hello world: 2597
hello world: 2598
hello world: 2599
hello world: 2600
hello world: 2601
hello world: 2602
hello world: 2603
hello world: 2604
hello world: 2605
hello world: 2606
hello world: 2607
hello world: 2608
hello world: 2609
hello world: 2610
hello world: 2611
hello world: 2612
hello world: 2613
hello world: 2614
hello world: 2615
hello world: 2616
hello world: 2617
hello world: 2618
hello world: 2619
hello world: 2620
hello world: 2621
hello world: 2622
hello world: 2623
hello world: 2624
hello world: 2625
hello world: 2626
hello world: 2627
hello world: 2628
hello world: 2629
hello world: 2630
hello world: 2631
hello world: 2632
hello world: 2633
hello world: 2634
hello world: 2635
hello world: 2636
hello world: 2637
hello world: 2638
hello world: 2639
hello world: 2640
hello world: 2641
hello world: 2642
hello world: 2643
hello world: 2644
hello world: 2645
hello world: 2646
hello world: 2647
hello world: 2648
hello world: 2649
hello world: 2650
hello world: 2651
hello world: 2652
hello world: 2653
hello world: 2654
hello world: 2655
hello world: 2656
hello world: 2657
hello world: 2658
hello world: 2659
hello world: 2660
hello world: 2661
hello world: 2662
hello world: 2663
hello world: 2664
hello world: 2665
hello world: 2666
hello world: 2667
hello world: 2668
hello world: 2669
hello world: 2670
hello world: 2671
hello world: 2672
hello world: 2673
hello world: 2674
hello world: 2675
hello world: 2676
hello world: 2677
hello world: 2678
hello world: 2679
hello world: 2680
hello world: 2681
hello world: 2682
hello world: 2683
hello world: 2684
hello world: 2685
hello world: 2686
hello world: 2687
hello world: 2688
hello world: 2689
hello world: 2690
hello world: 2691
hello world: 2692
hello world: 2693
hello world: 2694
hello world: 2695
hello world: 2696
hello world: 2697
hello world: 2698
hello world: 2699
hello world: 2700
hello world: 2701
hello world: 2702
hello world: 2703
hello world: 2704
hello world: 2705
hello world: 2706
hello world: 2707
hello world: 2708
hello world: 2709
hello world: 2710
hello world: 2711
hello world: 2712
hello world: 2713
hello world: 2714
hello world: 2715
hello world: 2716
hello world: 2717
hello world: 2718
hello world: 2719
hello world: 2720
hello world: 2721
hello world: 2722
hello world: 2723
hello world: 2724
hello world: 2725
hello world: 2726
hello world: 2727
hello world: 2728
hello world: 2729
hello world: 2730
hello world: 2731
hello world: 2732
hello world: 2733
hello world: 2734
hello world: 2735
hello world: 2736
hello world: 2737
hello world: 2738
hello world: 2739
hello world: 2740
hello world: 2741
hello world: 2742
hello world: 2743
hello world: 2744
hello world: 2745
hello world: 2746
hello world: 2747
hello world: 2748
hello world: 2749
hello world: 2750
hello world: 2751
hello world: 2752
hello world: 2753
hello world: 2754
hello world: 2755
hello world: 2756
hello world: 2757
hello world: 2758
hello world: 2759
hello world: 2760
hello world: 2761
hello world: 2762
hello world: 2763
hello world: 2764
hello world: 2765
hello world: 2766
hello world: 2767
hello world: 2768
hello world: 2769
hello world: 2770
hello world: 2771
hello world: 2772
hello world: 2773
hello world: 2774
hello world: 2775
hello world: 2776
hello world: 2777
hello world: 2778
hello world: 2779
hello world: 2780
hello world: 2781
hello world: 2782
hello world: 2783
hello world: 2784
hello world: 2785
hello world: 2786
hello world: 2787
hello world: 2788
hello world: 2789
hello world: 2790
hello world: 2791
hello 

```

In [19]:

```py
import os
os.remove('newfile.txt')

```

出现异常时候的读写：

In [20]:

```py
f = open('newfile.txt','w')
for i in range(3000):
    x = 1.0 / (i - 1000)
    f.write('hello world: ' + str(i) + '\n')

```

```py
---------------------------------------------------------------------------
ZeroDivisionError                         Traceback (most recent call last)
<ipython-input-20-8dc1cc9b29ec> in <module>()
 1 f = open('newfile.txt','w')
 2 for i in range(3000):
----> 3  x = 1.0 / (i - 1000)
 4     f.write('hello world: ' + str(i) + '\n')

ZeroDivisionError: float division by zero
```

查看已有内容：

In [21]:

```py
g = open('newfile.txt', 'r')
print g.read()
f.close()
g.close()

```

```py
hello world: 0
hello world: 1
hello world: 2
hello world: 3
hello world: 4
hello world: 5
hello world: 6
hello world: 7
hello world: 8
hello world: 9
hello world: 10
hello world: 11
hello world: 12
hello world: 13
hello world: 14
hello world: 15
hello world: 16
hello world: 17
hello world: 18
hello world: 19
hello world: 20
hello world: 21
hello world: 22
hello world: 23
hello world: 24
hello world: 25
hello world: 26
hello world: 27
hello world: 28
hello world: 29
hello world: 30
hello world: 31
hello world: 32
hello world: 33
hello world: 34
hello world: 35
hello world: 36
hello world: 37
hello world: 38
hello world: 39
hello world: 40
hello world: 41
hello world: 42
hello world: 43
hello world: 44
hello world: 45
hello world: 46
hello world: 47
hello world: 48
hello world: 49
hello world: 50
hello world: 51
hello world: 52
hello world: 53
hello world: 54
hello world: 55
hello world: 56
hello world: 57
hello world: 58
hello world: 59
hello world: 60
hello world: 61
hello world: 62
hello world: 63
hello world: 64
hello world: 65
hello world: 66
hello world: 67
hello world: 68
hello world: 69
hello world: 70
hello world: 71
hello world: 72
hello world: 73
hello world: 74
hello world: 75
hello world: 76
hello world: 77
hello world: 78
hello world: 79
hello world: 80
hello world: 81
hello world: 82
hello world: 83
hello world: 84
hello world: 85
hello world: 86
hello world: 87
hello world: 88
hello world: 89
hello world: 90
hello world: 91
hello world: 92
hello world: 93
hello world: 94
hello world: 95
hello world: 96
hello world: 97
hello world: 98
hello world: 99
hello world: 100
hello world: 101
hello world: 102
hello world: 103
hello world: 104
hello world: 105
hello world: 106
hello world: 107
hello world: 108
hello world: 109
hello world: 110
hello world: 111
hello world: 112
hello world: 113
hello world: 114
hello world: 115
hello world: 116
hello world: 117
hello world: 118
hello world: 119
hello world: 120
hello world: 121
hello world: 122
hello world: 123
hello world: 124
hello world: 125
hello world: 126
hello world: 127
hello world: 128
hello world: 129
hello world: 130
hello world: 131
hello world: 132
hello world: 133
hello world: 134
hello world: 135
hello world: 136
hello world: 137
hello world: 138
hello world: 139
hello world: 140
hello world: 141
hello world: 142
hello world: 143
hello world: 144
hello world: 145
hello world: 146
hello world: 147
hello world: 148
hello world: 149
hello world: 150
hello world: 151
hello world: 152
hello world: 153
hello world: 154
hello world: 155
hello world: 156
hello world: 157
hello world: 158
hello world: 159
hello world: 160
hello world: 161
hello world: 162
hello world: 163
hello world: 164
hello world: 165
hello world: 166
hello world: 167
hello world: 168
hello world: 169
hello world: 170
hello world: 171
hello world: 172
hello world: 173
hello world: 174
hello world: 175
hello world: 176
hello world: 177
hello world: 178
hello world: 179
hello world: 180
hello world: 181
hello world: 182
hello world: 183
hello world: 184
hello world: 185
hello world: 186
hello world: 187
hello world: 188
hello world: 189
hello world: 190
hello world: 191
hello world: 192
hello world: 193
hello world: 194
hello world: 195
hello world: 196
hello world: 197
hello world: 198
hello world: 199
hello world: 200
hello world: 201
hello world: 202
hello world: 203
hello world: 204
hello world: 205
hello world: 206
hello world: 207
hello world: 208
hello world: 209
hello world: 210
hello world: 211
hello world: 212
hello world: 213
hello world: 214
hello world: 215
hello world: 216
hello world: 217
hello world: 218
hello world: 219
hello world: 220
hello world: 221
hello world: 222
hello world: 223
hello world: 224
hello world: 225
hello world: 226
hello world: 227
hello world: 228
hello world: 229
hello world: 230
hello world: 231
hello world: 232
hello world: 233
hello world: 234
hello world: 235
hello world: 236
hello world: 237
hello world: 238
hello world: 239
hello world: 240
hello world: 241
hello world: 242
hello world: 243
hello world: 244
hello world: 245
hello world: 246
hello world: 247
hello world: 248
hello world: 249
hello world: 250
hello world: 251
hello world: 252
hello world: 253
hello world: 254
hello world: 255
hello world: 256
hello world: 257
hello world: 258
hello world: 259
hello world: 260
hello world: 261
hello world: 262
hello world: 263
hello world: 264
hello world: 265
hello world: 266
hello world: 267
hello world: 268
hello world: 269
hello world: 270
hello world: 271
hello world: 272
hello world: 273
hello world: 274
hello world: 275
hello world: 276
hello world: 277
hello world: 278
hello world: 279
hello world: 280
hello world: 281
hello world: 282
hello world: 283
hello world: 284
hello world: 285
hello world: 286
hello world: 287
hello world: 288
hello world: 289
hello world: 290
hello world: 291
hello world: 292
hello world: 293
hello world: 294
hello world: 295
hello world: 296
hello world: 297
hello world: 298
hello world: 299
hello world: 300
hello world: 301
hello world: 302
hello world: 303
hello world: 304
hello world: 305
hello world: 306
hello world: 307
hello world: 308
hello world: 309
hello world: 310
hello world: 311
hello world: 312
hello world: 313
hello world: 314
hello world: 315
hello world: 316
hello world: 317
hello world: 318
hello world: 319
hello world: 320
hello world: 321
hello world: 322
hello world: 323
hello world: 324
hello world: 325
hello world: 326
hello world: 327
hello world: 328
hello world: 329
hello world: 330
hello world: 331
hello world: 332
hello world: 333
hello world: 334
hello world: 335
hello world: 336
hello world: 337
hello world: 338
hello world: 339
hello world: 340
hello world: 341
hello world: 342
hello world: 343
hello world: 344
hello world: 345
hello world: 346
hello world: 347
hello world: 348
hello world: 349
hello world: 350
hello world: 351
hello world: 352
hello world: 353
hello world: 354
hello world: 355
hello world: 356
hello world: 357
hello world: 358
hello world: 359
hello world: 360
hello world: 361
hello world: 362
hello world: 363
hello world: 364
hello world: 365
hello world: 366
hello world: 367
hello world: 368
hello world: 369
hello world: 370
hello world: 371
hello world: 372
hello world: 373
hello world: 374
hello world: 375
hello world: 376
hello world: 377
hello world: 378
hello world: 379
hello world: 380
hello world: 381
hello world: 382
hello world: 383
hello world: 384
hello world: 385
hello world: 386
hello world: 387
hello world: 388
hello world: 389
hello world: 390
hello world: 391
hello world: 392
hello world: 393
hello world: 394
hello world: 395
hello world: 396
hello world: 397
hello world: 398
hello world: 399
hello world: 400
hello world: 401
hello world: 402
hello world: 403
hello world: 404
hello world: 405
hello world: 406
hello world: 407
hello world: 408
hello world: 409
hello world: 410
hello world: 411
hello world: 412
hello world: 413
hello world: 414
hello world: 415
hello world: 416
hello world: 417
hello world: 418
hello world: 419
hello world: 420
hello world: 421
hello world: 422
hello world: 423
hello world: 424
hello world: 425
hello world: 426
hello world: 427
hello world: 428
hello world: 429
hello world: 430
hello world: 431
hello world: 432
hello world: 433
hello world: 434
hello world: 435
hello world: 436
hello world: 437
hello world: 438
hello world: 439
hello world: 440
hello world: 441
hello world: 442
hello world: 443
hello world: 444
hello world: 445
hello world: 446
hello world: 447
hello world: 448
hello world: 449
hello world: 450
hello world: 451
hello world: 452
hello world: 453
hello world: 454
hello world: 455
hello world: 456
hello world: 457
hello world: 458
hello world: 459
hello world: 460
hello world: 461
hello world: 462
hello world: 463
hello world: 464
hello world: 465
hello world: 466
hello world: 467
hello world: 468
hello world: 469
hello world: 470
hello world: 471
hello world: 472
hello world: 473
hello world: 474
hello world: 475
hello world: 476
hello world: 477
hello world: 478
hello world: 479
hello world: 480
hello world: 481
hello world: 482
hello world: 483
hello world: 484
hello world: 485
hello world: 486
hello world: 487
hello world: 488
hello world: 489
hello world: 490
hello world: 491
hello world: 492
hello world: 493
hello world: 494
hello world: 495
hello world: 496
hello world: 497
hello world: 498
hello world: 499
hello world: 500
hello world: 501
hello world: 502
hello world: 503
hello world: 504
hello world: 505
hello world: 506
hello world: 507
hello world: 508
hello world: 509
hello world: 510
hello world: 511
hello world: 512
hello world: 513
hello world: 514
hello world: 515
hello world: 516
hello world: 517
hello world: 518
hello world: 519
hello world: 520
hello world: 521
hello world: 522
hello world: 523
hello world: 524
hello world: 525
hello world: 526
hello world: 527
hello world: 528
hello world: 529
hello world: 530
hello world: 531
hello world: 532
hello world: 533
hello world: 534
hello world: 535
hello world: 536
hello world: 537
hello world: 538
hello world: 539
hello world: 540
hello world: 541
hello world: 542
hello world: 543
hello world: 544
hello world: 545
hello world: 546
hello world: 547
hello world: 548
hello world: 549
hello world: 550
hello world: 551
hello world: 552
hello world: 553
hello world: 554
hello world: 555
hello world: 556
hello world: 557
hello world: 558
hello world: 559
hello world: 560
hello world: 561
hello world: 562
hello world: 563
hello world: 564
hello world: 565
hello world: 566
hello world: 567
hello world: 568
hello world: 569
hello world: 570
hello world: 571
hello world: 572
hello world: 573
hello world: 574
hello world: 575
hello world: 576
hello world: 577
hello world: 578
hello world: 579
hello world: 580
hello world: 581
hello world: 582
hello world: 583
hello world: 584
hello world: 585
hello world: 586
hello world: 587
hello world: 588
hello world: 589
hello world: 590
hello world: 591
hello world: 592
hello world: 593
hello world: 594
hello world: 595
hello world: 596
hello world: 597
hello world: 598
hello world: 599
hello world: 600
hello world: 601
hello world: 602
hello world: 603
hello world: 604
hello world: 605
hello world: 606
hello world: 607
hello world: 608
hello world: 609
hello world: 610
hello world: 611
hello world: 612
hello world: 613
hello world: 614
hello world: 615
hello world: 616
hello world: 617
hello world: 618
hello world: 619
hello world: 620
hello world: 621
hello world: 622
hello world: 623
hello world: 624
hello world: 625
hello world: 626
hello world: 627
hello world: 628
hello world: 629
hello world: 630
hello world: 631
hello world: 632
hello world: 633
hello world: 634
hello world: 635
hello world: 636
hello world: 637
hello world: 638
hello world: 639
hello world: 640
hello world: 641
hello world: 642
hello world: 643
hello world: 644
hello world: 645
hello world: 646
hello world: 647
hello world: 648
hello world: 649
hello world: 650
hello world: 651
hello world: 652
hello world: 653
hello world: 654
hello world: 655
hello world: 656
hello world: 657
hello world: 658
hello world: 659
hello world: 660
hello world: 661
hello world: 662
hello world: 663
hello world: 664
hello world: 665
hello world: 666
hello world: 667
hello world: 668
hello world: 669
hello world: 670
hello world: 671
hello world: 672
hello world: 673
hello world: 674
hello world: 675
hello world: 676
hello world: 677
hello world: 678
hello world: 679
hello world: 680
hello world: 681
hello world: 682
hello world: 683
hello world: 684
hello world: 685
hello world: 686
hello world: 687
hello world: 688
hello world: 689
hello world: 690
hello world: 691
hello world: 692
hello world: 693
hello world: 694
hello world: 695
hello world: 696
hello world: 697
hello world: 698
hello world: 699
hello world: 700
hello world: 701
hello world: 702
hello world: 703
hello world: 704
hello world: 705
hello world: 706
hello world: 707
hello world: 708
hello world: 709
hello world: 710
hello world: 711
hello world: 712
hello world: 713
hello world: 714
hello world: 715
hello world: 716
hello world: 717
hello world: 718
hello world: 719
hello world: 720
hello world: 721
hello world: 722
hello world: 723
hello world: 724
hello world: 725
hello world: 726
hello world: 727
hello world: 728
hello world: 729
hello world: 730
hello world: 731
hello world: 732
hello world: 733
hello world: 734
hello world: 735
hello world: 736
hello world: 737
hello world: 738
hello world: 739
hello world: 740
hello world: 741
hello world: 742
hello world: 743
hello world: 744
hello world: 745
hello world: 746
hello world: 747
hello world: 748
hello world: 749
hello world: 750
hello world: 751
hello world: 752
hello world: 753
hello world: 754
hello world: 755
hello world: 756
hello world: 757
hello world: 758
hello world: 759
hello world: 760
hello world: 761
hello world: 762
hello world: 763
hello world: 764
hello world: 765
hello world: 766
hello world: 767
hello world: 768
hello world: 769
hello world: 770
hello world: 771
hello world: 772
hello world: 773
hello world: 774
hello world: 775
hello world: 776
hello world: 777
hello world: 778
hello world: 779
hello world: 780
hello world: 781
hello world: 782
hello world: 783
hello world: 784
hello world: 785
hello world: 786
hello world: 787
hello world: 788
hello world: 789
hello world: 790
hello world: 791
hello world: 792
hello world: 793
hello world: 794
hello world: 795
hello world: 796
hello world: 797
hello world: 798
hello world: 799
hello world: 800
hello world: 801
hello world: 802
hello world: 803
hello world: 804
hello world: 805
hello world: 806
hello world: 807
hello world: 808
hello world: 809
hello world: 810
hello world: 811
hello world: 812
hello world: 813
hello world: 814
hello world: 815
hello world: 816
hello world: 817
hello world: 818
hello world: 819
hello world: 820
hello world: 821
hello world: 822
hello world: 823
hello world: 824
hello world: 825
hello world: 826
hello world: 827
hello world: 828
hello world: 829
hello world: 830
hello world: 831
hello world: 832
hello world: 833
hello world: 834
hello world: 835
hello world: 836
hello world: 837
hello world: 838
hello world: 839
hello world: 840
hello world: 841
hello world: 842
hello world: 843
hello world: 844
hello world: 845
hello world: 846
hello world: 847
hello world: 848
hello world: 849
hello world: 850
hello world: 851
hello world: 852
hello world: 853
hello world: 854
hello world: 855
hello world: 856
hello world: 857
hello world: 858
hello world: 859
hello world: 860
hello world: 861
hello world: 862
hello world: 863
hello world: 864
hello world: 865
hello world: 866
hello world: 867
hello world: 868
hello world: 869
hello world: 870
hello world: 871
hello world: 872
hello world: 873
hello world: 874
hello world: 875
hello world: 876
hello world: 877
hello world: 878
hello world: 879
hello world: 880
hello world: 881
hello world: 882
hello world: 883
hello world: 884
hello world: 885
hello world: 886
hello world: 887
hello world: 888
hello world: 889
hello world: 890
hello world: 891
hello world: 892
hello world: 893
hello world: 894
hello world: 895
hello world: 896
hello world: 897
hello world: 898
hello world: 899
hello world: 900
hello world: 901
hello world: 902
hello world: 903
hello world: 904
hello world: 905
hello world: 906
hello world: 907
hello world: 908
hello world: 909
hello world: 910
hello world: 911
hello world: 912
hello world: 913
hello world: 914
hello world: 915
hello world: 916
hello world: 917
hello world: 918
hello world: 919
hello world: 920
hello world: 921
hello world: 922
hello world: 923
hello world: 924
hello world: 925
hello world: 926
hello world: 927
hello world: 928
hello world: 929
hello world: 930
hello world: 931
hello world: 932
hello world: 933
hello world: 934
hello world: 935
hello world: 936
hello world: 937
hello world: 938
hello world: 939
hello world: 940
hello world: 941
hello world: 942
hello world: 943
hello world: 944
hello world: 945
hello world: 946
hello world: 947
hello world: 948
hello world: 949
hello world: 950
hello world: 951
hello world: 952
hello world: 953
hello world: 954
hello world: 955
hello world: 956
hello world: 957
hello world: 958
hello world: 959
hello world: 960
hello world: 961
hello world: 962
hello world: 963
hello world: 964
hello world: 965
hello world: 966
hello world: 967
hello world: 968
hello world: 969
hell

```

可以看到，出现异常的时候，磁盘的写入并没有完成，为此我们可以使用 `try/except/finally` 块来关闭文件，这里 `finally` 确保关闭文件，所有的写入已经完成。

In [22]:

```py
f = open('newfile.txt','w')
try:
    for i in range(3000):
        x = 1.0 / (i - 1000)
        f.write('hello world: ' + str(i) + '\n')
except Exception:
    print "something bad happened"
finally:
    f.close()

```

```py
something bad happened

```

In [23]:

```py
g = open('newfile.txt', 'r')
print g.read()
g.close()

```

```py
hello world: 0
hello world: 1
hello world: 2
hello world: 3
hello world: 4
hello world: 5
hello world: 6
hello world: 7
hello world: 8
hello world: 9
hello world: 10
hello world: 11
hello world: 12
hello world: 13
hello world: 14
hello world: 15
hello world: 16
hello world: 17
hello world: 18
hello world: 19
hello world: 20
hello world: 21
hello world: 22
hello world: 23
hello world: 24
hello world: 25
hello world: 26
hello world: 27
hello world: 28
hello world: 29
hello world: 30
hello world: 31
hello world: 32
hello world: 33
hello world: 34
hello world: 35
hello world: 36
hello world: 37
hello world: 38
hello world: 39
hello world: 40
hello world: 41
hello world: 42
hello world: 43
hello world: 44
hello world: 45
hello world: 46
hello world: 47
hello world: 48
hello world: 49
hello world: 50
hello world: 51
hello world: 52
hello world: 53
hello world: 54
hello world: 55
hello world: 56
hello world: 57
hello world: 58
hello world: 59
hello world: 60
hello world: 61
hello world: 62
hello world: 63
hello world: 64
hello world: 65
hello world: 66
hello world: 67
hello world: 68
hello world: 69
hello world: 70
hello world: 71
hello world: 72
hello world: 73
hello world: 74
hello world: 75
hello world: 76
hello world: 77
hello world: 78
hello world: 79
hello world: 80
hello world: 81
hello world: 82
hello world: 83
hello world: 84
hello world: 85
hello world: 86
hello world: 87
hello world: 88
hello world: 89
hello world: 90
hello world: 91
hello world: 92
hello world: 93
hello world: 94
hello world: 95
hello world: 96
hello world: 97
hello world: 98
hello world: 99
hello world: 100
hello world: 101
hello world: 102
hello world: 103
hello world: 104
hello world: 105
hello world: 106
hello world: 107
hello world: 108
hello world: 109
hello world: 110
hello world: 111
hello world: 112
hello world: 113
hello world: 114
hello world: 115
hello world: 116
hello world: 117
hello world: 118
hello world: 119
hello world: 120
hello world: 121
hello world: 122
hello world: 123
hello world: 124
hello world: 125
hello world: 126
hello world: 127
hello world: 128
hello world: 129
hello world: 130
hello world: 131
hello world: 132
hello world: 133
hello world: 134
hello world: 135
hello world: 136
hello world: 137
hello world: 138
hello world: 139
hello world: 140
hello world: 141
hello world: 142
hello world: 143
hello world: 144
hello world: 145
hello world: 146
hello world: 147
hello world: 148
hello world: 149
hello world: 150
hello world: 151
hello world: 152
hello world: 153
hello world: 154
hello world: 155
hello world: 156
hello world: 157
hello world: 158
hello world: 159
hello world: 160
hello world: 161
hello world: 162
hello world: 163
hello world: 164
hello world: 165
hello world: 166
hello world: 167
hello world: 168
hello world: 169
hello world: 170
hello world: 171
hello world: 172
hello world: 173
hello world: 174
hello world: 175
hello world: 176
hello world: 177
hello world: 178
hello world: 179
hello world: 180
hello world: 181
hello world: 182
hello world: 183
hello world: 184
hello world: 185
hello world: 186
hello world: 187
hello world: 188
hello world: 189
hello world: 190
hello world: 191
hello world: 192
hello world: 193
hello world: 194
hello world: 195
hello world: 196
hello world: 197
hello world: 198
hello world: 199
hello world: 200
hello world: 201
hello world: 202
hello world: 203
hello world: 204
hello world: 205
hello world: 206
hello world: 207
hello world: 208
hello world: 209
hello world: 210
hello world: 211
hello world: 212
hello world: 213
hello world: 214
hello world: 215
hello world: 216
hello world: 217
hello world: 218
hello world: 219
hello world: 220
hello world: 221
hello world: 222
hello world: 223
hello world: 224
hello world: 225
hello world: 226
hello world: 227
hello world: 228
hello world: 229
hello world: 230
hello world: 231
hello world: 232
hello world: 233
hello world: 234
hello world: 235
hello world: 236
hello world: 237
hello world: 238
hello world: 239
hello world: 240
hello world: 241
hello world: 242
hello world: 243
hello world: 244
hello world: 245
hello world: 246
hello world: 247
hello world: 248
hello world: 249
hello world: 250
hello world: 251
hello world: 252
hello world: 253
hello world: 254
hello world: 255
hello world: 256
hello world: 257
hello world: 258
hello world: 259
hello world: 260
hello world: 261
hello world: 262
hello world: 263
hello world: 264
hello world: 265
hello world: 266
hello world: 267
hello world: 268
hello world: 269
hello world: 270
hello world: 271
hello world: 272
hello world: 273
hello world: 274
hello world: 275
hello world: 276
hello world: 277
hello world: 278
hello world: 279
hello world: 280
hello world: 281
hello world: 282
hello world: 283
hello world: 284
hello world: 285
hello world: 286
hello world: 287
hello world: 288
hello world: 289
hello world: 290
hello world: 291
hello world: 292
hello world: 293
hello world: 294
hello world: 295
hello world: 296
hello world: 297
hello world: 298
hello world: 299
hello world: 300
hello world: 301
hello world: 302
hello world: 303
hello world: 304
hello world: 305
hello world: 306
hello world: 307
hello world: 308
hello world: 309
hello world: 310
hello world: 311
hello world: 312
hello world: 313
hello world: 314
hello world: 315
hello world: 316
hello world: 317
hello world: 318
hello world: 319
hello world: 320
hello world: 321
hello world: 322
hello world: 323
hello world: 324
hello world: 325
hello world: 326
hello world: 327
hello world: 328
hello world: 329
hello world: 330
hello world: 331
hello world: 332
hello world: 333
hello world: 334
hello world: 335
hello world: 336
hello world: 337
hello world: 338
hello world: 339
hello world: 340
hello world: 341
hello world: 342
hello world: 343
hello world: 344
hello world: 345
hello world: 346
hello world: 347
hello world: 348
hello world: 349
hello world: 350
hello world: 351
hello world: 352
hello world: 353
hello world: 354
hello world: 355
hello world: 356
hello world: 357
hello world: 358
hello world: 359
hello world: 360
hello world: 361
hello world: 362
hello world: 363
hello world: 364
hello world: 365
hello world: 366
hello world: 367
hello world: 368
hello world: 369
hello world: 370
hello world: 371
hello world: 372
hello world: 373
hello world: 374
hello world: 375
hello world: 376
hello world: 377
hello world: 378
hello world: 379
hello world: 380
hello world: 381
hello world: 382
hello world: 383
hello world: 384
hello world: 385
hello world: 386
hello world: 387
hello world: 388
hello world: 389
hello world: 390
hello world: 391
hello world: 392
hello world: 393
hello world: 394
hello world: 395
hello world: 396
hello world: 397
hello world: 398
hello world: 399
hello world: 400
hello world: 401
hello world: 402
hello world: 403
hello world: 404
hello world: 405
hello world: 406
hello world: 407
hello world: 408
hello world: 409
hello world: 410
hello world: 411
hello world: 412
hello world: 413
hello world: 414
hello world: 415
hello world: 416
hello world: 417
hello world: 418
hello world: 419
hello world: 420
hello world: 421
hello world: 422
hello world: 423
hello world: 424
hello world: 425
hello world: 426
hello world: 427
hello world: 428
hello world: 429
hello world: 430
hello world: 431
hello world: 432
hello world: 433
hello world: 434
hello world: 435
hello world: 436
hello world: 437
hello world: 438
hello world: 439
hello world: 440
hello world: 441
hello world: 442
hello world: 443
hello world: 444
hello world: 445
hello world: 446
hello world: 447
hello world: 448
hello world: 449
hello world: 450
hello world: 451
hello world: 452
hello world: 453
hello world: 454
hello world: 455
hello world: 456
hello world: 457
hello world: 458
hello world: 459
hello world: 460
hello world: 461
hello world: 462
hello world: 463
hello world: 464
hello world: 465
hello world: 466
hello world: 467
hello world: 468
hello world: 469
hello world: 470
hello world: 471
hello world: 472
hello world: 473
hello world: 474
hello world: 475
hello world: 476
hello world: 477
hello world: 478
hello world: 479
hello world: 480
hello world: 481
hello world: 482
hello world: 483
hello world: 484
hello world: 485
hello world: 486
hello world: 487
hello world: 488
hello world: 489
hello world: 490
hello world: 491
hello world: 492
hello world: 493
hello world: 494
hello world: 495
hello world: 496
hello world: 497
hello world: 498
hello world: 499
hello world: 500
hello world: 501
hello world: 502
hello world: 503
hello world: 504
hello world: 505
hello world: 506
hello world: 507
hello world: 508
hello world: 509
hello world: 510
hello world: 511
hello world: 512
hello world: 513
hello world: 514
hello world: 515
hello world: 516
hello world: 517
hello world: 518
hello world: 519
hello world: 520
hello world: 521
hello world: 522
hello world: 523
hello world: 524
hello world: 525
hello world: 526
hello world: 527
hello world: 528
hello world: 529
hello world: 530
hello world: 531
hello world: 532
hello world: 533
hello world: 534
hello world: 535
hello world: 536
hello world: 537
hello world: 538
hello world: 539
hello world: 540
hello world: 541
hello world: 542
hello world: 543
hello world: 544
hello world: 545
hello world: 546
hello world: 547
hello world: 548
hello world: 549
hello world: 550
hello world: 551
hello world: 552
hello world: 553
hello world: 554
hello world: 555
hello world: 556
hello world: 557
hello world: 558
hello world: 559
hello world: 560
hello world: 561
hello world: 562
hello world: 563
hello world: 564
hello world: 565
hello world: 566
hello world: 567
hello world: 568
hello world: 569
hello world: 570
hello world: 571
hello world: 572
hello world: 573
hello world: 574
hello world: 575
hello world: 576
hello world: 577
hello world: 578
hello world: 579
hello world: 580
hello world: 581
hello world: 582
hello world: 583
hello world: 584
hello world: 585
hello world: 586
hello world: 587
hello world: 588
hello world: 589
hello world: 590
hello world: 591
hello world: 592
hello world: 593
hello world: 594
hello world: 595
hello world: 596
hello world: 597
hello world: 598
hello world: 599
hello world: 600
hello world: 601
hello world: 602
hello world: 603
hello world: 604
hello world: 605
hello world: 606
hello world: 607
hello world: 608
hello world: 609
hello world: 610
hello world: 611
hello world: 612
hello world: 613
hello world: 614
hello world: 615
hello world: 616
hello world: 617
hello world: 618
hello world: 619
hello world: 620
hello world: 621
hello world: 622
hello world: 623
hello world: 624
hello world: 625
hello world: 626
hello world: 627
hello world: 628
hello world: 629
hello world: 630
hello world: 631
hello world: 632
hello world: 633
hello world: 634
hello world: 635
hello world: 636
hello world: 637
hello world: 638
hello world: 639
hello world: 640
hello world: 641
hello world: 642
hello world: 643
hello world: 644
hello world: 645
hello world: 646
hello world: 647
hello world: 648
hello world: 649
hello world: 650
hello world: 651
hello world: 652
hello world: 653
hello world: 654
hello world: 655
hello world: 656
hello world: 657
hello world: 658
hello world: 659
hello world: 660
hello world: 661
hello world: 662
hello world: 663
hello world: 664
hello world: 665
hello world: 666
hello world: 667
hello world: 668
hello world: 669
hello world: 670
hello world: 671
hello world: 672
hello world: 673
hello world: 674
hello world: 675
hello world: 676
hello world: 677
hello world: 678
hello world: 679
hello world: 680
hello world: 681
hello world: 682
hello world: 683
hello world: 684
hello world: 685
hello world: 686
hello world: 687
hello world: 688
hello world: 689
hello world: 690
hello world: 691
hello world: 692
hello world: 693
hello world: 694
hello world: 695
hello world: 696
hello world: 697
hello world: 698
hello world: 699
hello world: 700
hello world: 701
hello world: 702
hello world: 703
hello world: 704
hello world: 705
hello world: 706
hello world: 707
hello world: 708
hello world: 709
hello world: 710
hello world: 711
hello world: 712
hello world: 713
hello world: 714
hello world: 715
hello world: 716
hello world: 717
hello world: 718
hello world: 719
hello world: 720
hello world: 721
hello world: 722
hello world: 723
hello world: 724
hello world: 725
hello world: 726
hello world: 727
hello world: 728
hello world: 729
hello world: 730
hello world: 731
hello world: 732
hello world: 733
hello world: 734
hello world: 735
hello world: 736
hello world: 737
hello world: 738
hello world: 739
hello world: 740
hello world: 741
hello world: 742
hello world: 743
hello world: 744
hello world: 745
hello world: 746
hello world: 747
hello world: 748
hello world: 749
hello world: 750
hello world: 751
hello world: 752
hello world: 753
hello world: 754
hello world: 755
hello world: 756
hello world: 757
hello world: 758
hello world: 759
hello world: 760
hello world: 761
hello world: 762
hello world: 763
hello world: 764
hello world: 765
hello world: 766
hello world: 767
hello world: 768
hello world: 769
hello world: 770
hello world: 771
hello world: 772
hello world: 773
hello world: 774
hello world: 775
hello world: 776
hello world: 777
hello world: 778
hello world: 779
hello world: 780
hello world: 781
hello world: 782
hello world: 783
hello world: 784
hello world: 785
hello world: 786
hello world: 787
hello world: 788
hello world: 789
hello world: 790
hello world: 791
hello world: 792
hello world: 793
hello world: 794
hello world: 795
hello world: 796
hello world: 797
hello world: 798
hello world: 799
hello world: 800
hello world: 801
hello world: 802
hello world: 803
hello world: 804
hello world: 805
hello world: 806
hello world: 807
hello world: 808
hello world: 809
hello world: 810
hello world: 811
hello world: 812
hello world: 813
hello world: 814
hello world: 815
hello world: 816
hello world: 817
hello world: 818
hello world: 819
hello world: 820
hello world: 821
hello world: 822
hello world: 823
hello world: 824
hello world: 825
hello world: 826
hello world: 827
hello world: 828
hello world: 829
hello world: 830
hello world: 831
hello world: 832
hello world: 833
hello world: 834
hello world: 835
hello world: 836
hello world: 837
hello world: 838
hello world: 839
hello world: 840
hello world: 841
hello world: 842
hello world: 843
hello world: 844
hello world: 845
hello world: 846
hello world: 847
hello world: 848
hello world: 849
hello world: 850
hello world: 851
hello world: 852
hello world: 853
hello world: 854
hello world: 855
hello world: 856
hello world: 857
hello world: 858
hello world: 859
hello world: 860
hello world: 861
hello world: 862
hello world: 863
hello world: 864
hello world: 865
hello world: 866
hello world: 867
hello world: 868
hello world: 869
hello world: 870
hello world: 871
hello world: 872
hello world: 873
hello world: 874
hello world: 875
hello world: 876
hello world: 877
hello world: 878
hello world: 879
hello world: 880
hello world: 881
hello world: 882
hello world: 883
hello world: 884
hello world: 885
hello world: 886
hello world: 887
hello world: 888
hello world: 889
hello world: 890
hello world: 891
hello world: 892
hello world: 893
hello world: 894
hello world: 895
hello world: 896
hello world: 897
hello world: 898
hello world: 899
hello world: 900
hello world: 901
hello world: 902
hello world: 903
hello world: 904
hello world: 905
hello world: 906
hello world: 907
hello world: 908
hello world: 909
hello world: 910
hello world: 911
hello world: 912
hello world: 913
hello world: 914
hello world: 915
hello world: 916
hello world: 917
hello world: 918
hello world: 919
hello world: 920
hello world: 921
hello world: 922
hello world: 923
hello world: 924
hello world: 925
hello world: 926
hello world: 927
hello world: 928
hello world: 929
hello world: 930
hello world: 931
hello world: 932
hello world: 933
hello world: 934
hello world: 935
hello world: 936
hello world: 937
hello world: 938
hello world: 939
hello world: 940
hello world: 941
hello world: 942
hello world: 943
hello world: 944
hello world: 945
hello world: 946
hello world: 947
hello world: 948
hello world: 949
hello world: 950
hello world: 951
hello world: 952
hello world: 953
hello world: 954
hello world: 955
hello world: 956
hello world: 957
hello world: 958
hello world: 959
hello world: 960
hello world: 961
hello world: 962
hello world: 963
hello world: 964
hello world: 965
hello world: 966
hello world: 967
hello world: 968
hello world: 969
hello world: 970
hello world: 971
hello world: 972
hello world: 973
hello world: 974
hello world: 975
hello world: 976
hello world: 977
hello world: 978
hello world: 979
hello world: 980
hello world: 981
hello world: 982
hello world: 983
hello world: 984
hello world: 985
hello world: 986
hello world: 987
hello world: 988
hello world: 989
hello world: 990
hello world: 991
hello world: 992
hello world: 993
hello world: 994
hello world: 995
hello world: 996
hello world: 997
hello world: 998
hello world: 999

```

## with 方法

事实上，**Python**提供了更安全的方法，当 `with` 块的内容结束后，**Python**会自动调用它的`close` 方法，确保读写的安全：

In [24]:

```py
with open('newfile.txt','w') as f:
    for i in range(3000):
        x = 1.0 / (i - 1000)
        f.write('hello world: ' + str(i) + '\n')

```

```py
---------------------------------------------------------------------------
ZeroDivisionError                         Traceback (most recent call last)
<ipython-input-24-9d2a70065b27> in <module>()
 1 with open('newfile.txt','w') as f:
 2     for i in range(3000):
----> 3  x = 1.0 / (i - 1000)
 4         f.write('hello world: ' + str(i) + '\n')

ZeroDivisionError: float division by zero
```

与 `try/exception/finally` 效果相同，但更简单。

In [25]:

```py
g = open('newfile.txt', 'r')
print g.read()
g.close()

```

```py
hello world: 0
hello world: 1
hello world: 2
hello world: 3
hello world: 4
hello world: 5
hello world: 6
hello world: 7
hello world: 8
hello world: 9
hello world: 10
hello world: 11
hello world: 12
hello world: 13
hello world: 14
hello world: 15
hello world: 16
hello world: 17
hello world: 18
hello world: 19
hello world: 20
hello world: 21
hello world: 22
hello world: 23
hello world: 24
hello world: 25
hello world: 26
hello world: 27
hello world: 28
hello world: 29
hello world: 30
hello world: 31
hello world: 32
hello world: 33
hello world: 34
hello world: 35
hello world: 36
hello world: 37
hello world: 38
hello world: 39
hello world: 40
hello world: 41
hello world: 42
hello world: 43
hello world: 44
hello world: 45
hello world: 46
hello world: 47
hello world: 48
hello world: 49
hello world: 50
hello world: 51
hello world: 52
hello world: 53
hello world: 54
hello world: 55
hello world: 56
hello world: 57
hello world: 58
hello world: 59
hello world: 60
hello world: 61
hello world: 62
hello world: 63
hello world: 64
hello world: 65
hello world: 66
hello world: 67
hello world: 68
hello world: 69
hello world: 70
hello world: 71
hello world: 72
hello world: 73
hello world: 74
hello world: 75
hello world: 76
hello world: 77
hello world: 78
hello world: 79
hello world: 80
hello world: 81
hello world: 82
hello world: 83
hello world: 84
hello world: 85
hello world: 86
hello world: 87
hello world: 88
hello world: 89
hello world: 90
hello world: 91
hello world: 92
hello world: 93
hello world: 94
hello world: 95
hello world: 96
hello world: 97
hello world: 98
hello world: 99
hello world: 100
hello world: 101
hello world: 102
hello world: 103
hello world: 104
hello world: 105
hello world: 106
hello world: 107
hello world: 108
hello world: 109
hello world: 110
hello world: 111
hello world: 112
hello world: 113
hello world: 114
hello world: 115
hello world: 116
hello world: 117
hello world: 118
hello world: 119
hello world: 120
hello world: 121
hello world: 122
hello world: 123
hello world: 124
hello world: 125
hello world: 126
hello world: 127
hello world: 128
hello world: 129
hello world: 130
hello world: 131
hello world: 132
hello world: 133
hello world: 134
hello world: 135
hello world: 136
hello world: 137
hello world: 138
hello world: 139
hello world: 140
hello world: 141
hello world: 142
hello world: 143
hello world: 144
hello world: 145
hello world: 146
hello world: 147
hello world: 148
hello world: 149
hello world: 150
hello world: 151
hello world: 152
hello world: 153
hello world: 154
hello world: 155
hello world: 156
hello world: 157
hello world: 158
hello world: 159
hello world: 160
hello world: 161
hello world: 162
hello world: 163
hello world: 164
hello world: 165
hello world: 166
hello world: 167
hello world: 168
hello world: 169
hello world: 170
hello world: 171
hello world: 172
hello world: 173
hello world: 174
hello world: 175
hello world: 176
hello world: 177
hello world: 178
hello world: 179
hello world: 180
hello world: 181
hello world: 182
hello world: 183
hello world: 184
hello world: 185
hello world: 186
hello world: 187
hello world: 188
hello world: 189
hello world: 190
hello world: 191
hello world: 192
hello world: 193
hello world: 194
hello world: 195
hello world: 196
hello world: 197
hello world: 198
hello world: 199
hello world: 200
hello world: 201
hello world: 202
hello world: 203
hello world: 204
hello world: 205
hello world: 206
hello world: 207
hello world: 208
hello world: 209
hello world: 210
hello world: 211
hello world: 212
hello world: 213
hello world: 214
hello world: 215
hello world: 216
hello world: 217
hello world: 218
hello world: 219
hello world: 220
hello world: 221
hello world: 222
hello world: 223
hello world: 224
hello world: 225
hello world: 226
hello world: 227
hello world: 228
hello world: 229
hello world: 230
hello world: 231
hello world: 232
hello world: 233
hello world: 234
hello world: 235
hello world: 236
hello world: 237
hello world: 238
hello world: 239
hello world: 240
hello world: 241
hello world: 242
hello world: 243
hello world: 244
hello world: 245
hello world: 246
hello world: 247
hello world: 248
hello world: 249
hello world: 250
hello world: 251
hello world: 252
hello world: 253
hello world: 254
hello world: 255
hello world: 256
hello world: 257
hello world: 258
hello world: 259
hello world: 260
hello world: 261
hello world: 262
hello world: 263
hello world: 264
hello world: 265
hello world: 266
hello world: 267
hello world: 268
hello world: 269
hello world: 270
hello world: 271
hello world: 272
hello world: 273
hello world: 274
hello world: 275
hello world: 276
hello world: 277
hello world: 278
hello world: 279
hello world: 280
hello world: 281
hello world: 282
hello world: 283
hello world: 284
hello world: 285
hello world: 286
hello world: 287
hello world: 288
hello world: 289
hello world: 290
hello world: 291
hello world: 292
hello world: 293
hello world: 294
hello world: 295
hello world: 296
hello world: 297
hello world: 298
hello world: 299
hello world: 300
hello world: 301
hello world: 302
hello world: 303
hello world: 304
hello world: 305
hello world: 306
hello world: 307
hello world: 308
hello world: 309
hello world: 310
hello world: 311
hello world: 312
hello world: 313
hello world: 314
hello world: 315
hello world: 316
hello world: 317
hello world: 318
hello world: 319
hello world: 320
hello world: 321
hello world: 322
hello world: 323
hello world: 324
hello world: 325
hello world: 326
hello world: 327
hello world: 328
hello world: 329
hello world: 330
hello world: 331
hello world: 332
hello world: 333
hello world: 334
hello world: 335
hello world: 336
hello world: 337
hello world: 338
hello world: 339
hello world: 340
hello world: 341
hello world: 342
hello world: 343
hello world: 344
hello world: 345
hello world: 346
hello world: 347
hello world: 348
hello world: 349
hello world: 350
hello world: 351
hello world: 352
hello world: 353
hello world: 354
hello world: 355
hello world: 356
hello world: 357
hello world: 358
hello world: 359
hello world: 360
hello world: 361
hello world: 362
hello world: 363
hello world: 364
hello world: 365
hello world: 366
hello world: 367
hello world: 368
hello world: 369
hello world: 370
hello world: 371
hello world: 372
hello world: 373
hello world: 374
hello world: 375
hello world: 376
hello world: 377
hello world: 378
hello world: 379
hello world: 380
hello world: 381
hello world: 382
hello world: 383
hello world: 384
hello world: 385
hello world: 386
hello world: 387
hello world: 388
hello world: 389
hello world: 390
hello world: 391
hello world: 392
hello world: 393
hello world: 394
hello world: 395
hello world: 396
hello world: 397
hello world: 398
hello world: 399
hello world: 400
hello world: 401
hello world: 402
hello world: 403
hello world: 404
hello world: 405
hello world: 406
hello world: 407
hello world: 408
hello world: 409
hello world: 410
hello world: 411
hello world: 412
hello world: 413
hello world: 414
hello world: 415
hello world: 416
hello world: 417
hello world: 418
hello world: 419
hello world: 420
hello world: 421
hello world: 422
hello world: 423
hello world: 424
hello world: 425
hello world: 426
hello world: 427
hello world: 428
hello world: 429
hello world: 430
hello world: 431
hello world: 432
hello world: 433
hello world: 434
hello world: 435
hello world: 436
hello world: 437
hello world: 438
hello world: 439
hello world: 440
hello world: 441
hello world: 442
hello world: 443
hello world: 444
hello world: 445
hello world: 446
hello world: 447
hello world: 448
hello world: 449
hello world: 450
hello world: 451
hello world: 452
hello world: 453
hello world: 454
hello world: 455
hello world: 456
hello world: 457
hello world: 458
hello world: 459
hello world: 460
hello world: 461
hello world: 462
hello world: 463
hello world: 464
hello world: 465
hello world: 466
hello world: 467
hello world: 468
hello world: 469
hello world: 470
hello world: 471
hello world: 472
hello world: 473
hello world: 474
hello world: 475
hello world: 476
hello world: 477
hello world: 478
hello world: 479
hello world: 480
hello world: 481
hello world: 482
hello world: 483
hello world: 484
hello world: 485
hello world: 486
hello world: 487
hello world: 488
hello world: 489
hello world: 490
hello world: 491
hello world: 492
hello world: 493
hello world: 494
hello world: 495
hello world: 496
hello world: 497
hello world: 498
hello world: 499
hello world: 500
hello world: 501
hello world: 502
hello world: 503
hello world: 504
hello world: 505
hello world: 506
hello world: 507
hello world: 508
hello world: 509
hello world: 510
hello world: 511
hello world: 512
hello world: 513
hello world: 514
hello world: 515
hello world: 516
hello world: 517
hello world: 518
hello world: 519
hello world: 520
hello world: 521
hello world: 522
hello world: 523
hello world: 524
hello world: 525
hello world: 526
hello world: 527
hello world: 528
hello world: 529
hello world: 530
hello world: 531
hello world: 532
hello world: 533
hello world: 534
hello world: 535
hello world: 536
hello world: 537
hello world: 538
hello world: 539
hello world: 540
hello world: 541
hello world: 542
hello world: 543
hello world: 544
hello world: 545
hello world: 546
hello world: 547
hello world: 548
hello world: 549
hello world: 550
hello world: 551
hello world: 552
hello world: 553
hello world: 554
hello world: 555
hello world: 556
hello world: 557
hello world: 558
hello world: 559
hello world: 560
hello world: 561
hello world: 562
hello world: 563
hello world: 564
hello world: 565
hello world: 566
hello world: 567
hello world: 568
hello world: 569
hello world: 570
hello world: 571
hello world: 572
hello world: 573
hello world: 574
hello world: 575
hello world: 576
hello world: 577
hello world: 578
hello world: 579
hello world: 580
hello world: 581
hello world: 582
hello world: 583
hello world: 584
hello world: 585
hello world: 586
hello world: 587
hello world: 588
hello world: 589
hello world: 590
hello world: 591
hello world: 592
hello world: 593
hello world: 594
hello world: 595
hello world: 596
hello world: 597
hello world: 598
hello world: 599
hello world: 600
hello world: 601
hello world: 602
hello world: 603
hello world: 604
hello world: 605
hello world: 606
hello world: 607
hello world: 608
hello world: 609
hello world: 610
hello world: 611
hello world: 612
hello world: 613
hello world: 614
hello world: 615
hello world: 616
hello world: 617
hello world: 618
hello world: 619
hello world: 620
hello world: 621
hello world: 622
hello world: 623
hello world: 624
hello world: 625
hello world: 626
hello world: 627
hello world: 628
hello world: 629
hello world: 630
hello world: 631
hello world: 632
hello world: 633
hello world: 634
hello world: 635
hello world: 636
hello world: 637
hello world: 638
hello world: 639
hello world: 640
hello world: 641
hello world: 642
hello world: 643
hello world: 644
hello world: 645
hello world: 646
hello world: 647
hello world: 648
hello world: 649
hello world: 650
hello world: 651
hello world: 652
hello world: 653
hello world: 654
hello world: 655
hello world: 656
hello world: 657
hello world: 658
hello world: 659
hello world: 660
hello world: 661
hello world: 662
hello world: 663
hello world: 664
hello world: 665
hello world: 666
hello world: 667
hello world: 668
hello world: 669
hello world: 670
hello world: 671
hello world: 672
hello world: 673
hello world: 674
hello world: 675
hello world: 676
hello world: 677
hello world: 678
hello world: 679
hello world: 680
hello world: 681
hello world: 682
hello world: 683
hello world: 684
hello world: 685
hello world: 686
hello world: 687
hello world: 688
hello world: 689
hello world: 690
hello world: 691
hello world: 692
hello world: 693
hello world: 694
hello world: 695
hello world: 696
hello world: 697
hello world: 698
hello world: 699
hello world: 700
hello world: 701
hello world: 702
hello world: 703
hello world: 704
hello world: 705
hello world: 706
hello world: 707
hello world: 708
hello world: 709
hello world: 710
hello world: 711
hello world: 712
hello world: 713
hello world: 714
hello world: 715
hello world: 716
hello world: 717
hello world: 718
hello world: 719
hello world: 720
hello world: 721
hello world: 722
hello world: 723
hello world: 724
hello world: 725
hello world: 726
hello world: 727
hello world: 728
hello world: 729
hello world: 730
hello world: 731
hello world: 732
hello world: 733
hello world: 734
hello world: 735
hello world: 736
hello world: 737
hello world: 738
hello world: 739
hello world: 740
hello world: 741
hello world: 742
hello world: 743
hello world: 744
hello world: 745
hello world: 746
hello world: 747
hello world: 748
hello world: 749
hello world: 750
hello world: 751
hello world: 752
hello world: 753
hello world: 754
hello world: 755
hello world: 756
hello world: 757
hello world: 758
hello world: 759
hello world: 760
hello world: 761
hello world: 762
hello world: 763
hello world: 764
hello world: 765
hello world: 766
hello world: 767
hello world: 768
hello world: 769
hello world: 770
hello world: 771
hello world: 772
hello world: 773
hello world: 774
hello world: 775
hello world: 776
hello world: 777
hello world: 778
hello world: 779
hello world: 780
hello world: 781
hello world: 782
hello world: 783
hello world: 784
hello world: 785
hello world: 786
hello world: 787
hello world: 788
hello world: 789
hello world: 790
hello world: 791
hello world: 792
hello world: 793
hello world: 794
hello world: 795
hello world: 796
hello world: 797
hello world: 798
hello world: 799
hello world: 800
hello world: 801
hello world: 802
hello world: 803
hello world: 804
hello world: 805
hello world: 806
hello world: 807
hello world: 808
hello world: 809
hello world: 810
hello world: 811
hello world: 812
hello world: 813
hello world: 814
hello world: 815
hello world: 816
hello world: 817
hello world: 818
hello world: 819
hello world: 820
hello world: 821
hello world: 822
hello world: 823
hello world: 824
hello world: 825
hello world: 826
hello world: 827
hello world: 828
hello world: 829
hello world: 830
hello world: 831
hello world: 832
hello world: 833
hello world: 834
hello world: 835
hello world: 836
hello world: 837
hello world: 838
hello world: 839
hello world: 840
hello world: 841
hello world: 842
hello world: 843
hello world: 844
hello world: 845
hello world: 846
hello world: 847
hello world: 848
hello world: 849
hello world: 850
hello world: 851
hello world: 852
hello world: 853
hello world: 854
hello world: 855
hello world: 856
hello world: 857
hello world: 858
hello world: 859
hello world: 860
hello world: 861
hello world: 862
hello world: 863
hello world: 864
hello world: 865
hello world: 866
hello world: 867
hello world: 868
hello world: 869
hello world: 870
hello world: 871
hello world: 872
hello world: 873
hello world: 874
hello world: 875
hello world: 876
hello world: 877
hello world: 878
hello world: 879
hello world: 880
hello world: 881
hello world: 882
hello world: 883
hello world: 884
hello world: 885
hello world: 886
hello world: 887
hello world: 888
hello world: 889
hello world: 890
hello world: 891
hello world: 892
hello world: 893
hello world: 894
hello world: 895
hello world: 896
hello world: 897
hello world: 898
hello world: 899
hello world: 900
hello world: 901
hello world: 902
hello world: 903
hello world: 904
hello world: 905
hello world: 906
hello world: 907
hello world: 908
hello world: 909
hello world: 910
hello world: 911
hello world: 912
hello world: 913
hello world: 914
hello world: 915
hello world: 916
hello world: 917
hello world: 918
hello world: 919
hello world: 920
hello world: 921
hello world: 922
hello world: 923
hello world: 924
hello world: 925
hello world: 926
hello world: 927
hello world: 928
hello world: 929
hello world: 930
hello world: 931
hello world: 932
hello world: 933
hello world: 934
hello world: 935
hello world: 936
hello world: 937
hello world: 938
hello world: 939
hello world: 940
hello world: 941
hello world: 942
hello world: 943
hello world: 944
hello world: 945
hello world: 946
hello world: 947
hello world: 948
hello world: 949
hello world: 950
hello world: 951
hello world: 952
hello world: 953
hello world: 954
hello world: 955
hello world: 956
hello world: 957
hello world: 958
hello world: 959
hello world: 960
hello world: 961
hello world: 962
hello world: 963
hello world: 964
hello world: 965
hello world: 966
hello world: 967
hello world: 968
hello world: 969
hello world: 970
hello world: 971
hello world: 972
hello world: 973
hello world: 974
hello world: 975
hello world: 976
hello world: 977
hello world: 978
hello world: 979
hello world: 980
hello world: 981
hello world: 982
hello world: 983
hello world: 984
hello world: 985
hello world: 986
hello world: 987
hello world: 988
hello world: 989
hello world: 990
hello world: 991
hello world: 992
hello world: 993
hello world: 994
hello world: 995
hello world: 996
hello world: 997
hello world: 998
hello world: 999

```

所以，写文件时候要确保文件被正确关闭。

In [26]:

```py
import os
os.remove('newfile.txt')

```